/********************************************************************************
** Form generated from reading UI file 'base_k3bexternalencoderconfigwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BASE_K3BEXTERNALENCODERCONFIGWIDGET_H
#define UI_BASE_K3BEXTERNALENCODERCONFIGWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_base_K3bExternalEncoderConfigWidget
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *label;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QTreeWidget *m_viewEncoders;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacer3;
    QPushButton *m_buttonRemove;
    QPushButton *m_buttonEdit;
    QPushButton *m_buttonAdd;

    void setupUi(QWidget *base_K3bExternalEncoderConfigWidget)
    {
        if (base_K3bExternalEncoderConfigWidget->objectName().isEmpty())
            base_K3bExternalEncoderConfigWidget->setObjectName("base_K3bExternalEncoderConfigWidget");
        base_K3bExternalEncoderConfigWidget->resize(493, 386);
        verticalLayout_2 = new QVBoxLayout(base_K3bExternalEncoderConfigWidget);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        label = new QLabel(base_K3bExternalEncoderConfigWidget);
        label->setObjectName("label");
        label->setWordWrap(true);

        verticalLayout_2->addWidget(label);

        groupBox = new QGroupBox(base_K3bExternalEncoderConfigWidget);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        m_viewEncoders = new QTreeWidget(groupBox);
        m_viewEncoders->setObjectName("m_viewEncoders");
        m_viewEncoders->setRootIsDecorated(false);
        m_viewEncoders->setAllColumnsShowFocus(true);

        verticalLayout->addWidget(m_viewEncoders);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        spacer3 = new QSpacerItem(71, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer3);

        m_buttonRemove = new QPushButton(groupBox);
        m_buttonRemove->setObjectName("m_buttonRemove");

        hboxLayout->addWidget(m_buttonRemove);

        m_buttonEdit = new QPushButton(groupBox);
        m_buttonEdit->setObjectName("m_buttonEdit");

        hboxLayout->addWidget(m_buttonEdit);

        m_buttonAdd = new QPushButton(groupBox);
        m_buttonAdd->setObjectName("m_buttonAdd");

        hboxLayout->addWidget(m_buttonAdd);


        verticalLayout->addLayout(hboxLayout);


        verticalLayout_2->addWidget(groupBox);


        retranslateUi(base_K3bExternalEncoderConfigWidget);

        QMetaObject::connectSlotsByName(base_K3bExternalEncoderConfigWidget);
    } // setupUi

    void retranslateUi(QWidget *base_K3bExternalEncoderConfigWidget)
    {
        label->setText(tr2i18n("<p>This dialog can be used to setup external command line applications as audio encoders. These can then be used by K3b to encode audio data (Tracks from an audio CD or the titles from an audio project) to formats that are normally not supported (i.e. no encoder plugin exists).\n"
"<p>K3b comes with a selection of predefined external applications that depends on the installed applications.", nullptr));
        groupBox->setTitle(tr2i18n("Configured Encoders", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = m_viewEncoders->headerItem();
        ___qtreewidgetitem->setText(2, tr2i18n("Command", nullptr));
        ___qtreewidgetitem->setText(1, tr2i18n("Extension", nullptr));
        ___qtreewidgetitem->setText(0, tr2i18n("Name", nullptr));
        m_buttonRemove->setText(tr2i18n("Remove", nullptr));
        m_buttonEdit->setText(tr2i18n("Edit...", nullptr));
        m_buttonAdd->setText(tr2i18n("Add...", nullptr));
        (void)base_K3bExternalEncoderConfigWidget;
    } // retranslateUi

};

namespace Ui {
    class base_K3bExternalEncoderConfigWidget: public Ui_base_K3bExternalEncoderConfigWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // BASE_K3BEXTERNALENCODERCONFIGWIDGET_H

