/****************************************************************************
** Meta object code from reading C++ file 'k3bgrowisofshandler.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/libk3b/projects/k3bgrowisofshandler.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3bgrowisofshandler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b16GrowisofsHandlerE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::GrowisofsHandler::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b16GrowisofsHandlerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::GrowisofsHandler",
        "infoMessage",
        "",
        "newSubTask",
        "buffer",
        "deviceBuffer",
        "flushingCache",
        "reset",
        "K3b::Device::Device*",
        "dao",
        "handleStart",
        "handleLine",
        "handleExit",
        "exitCode",
        "slotCheckBufferStatus",
        "slotCheckBufferStatusDone",
        "Device::DeviceHandler*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'infoMessage'
        QtMocHelpers::SignalData<void(const QString &, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { QMetaType::Int, 2 },
        }}),
        // Signal 'newSubTask'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'buffer'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'deviceBuffer'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'flushingCache'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reset'
        QtMocHelpers::SlotData<void(K3b::Device::Device *, bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 }, { QMetaType::Bool, 9 },
        }}),
        // Slot 'reset'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Slot 'reset'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'handleStart'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleLine'
        QtMocHelpers::SlotData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'handleExit'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Slot 'slotCheckBufferStatus'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCheckBufferStatusDone'
        QtMocHelpers::SlotData<void(Device::DeviceHandler *)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GrowisofsHandler, qt_meta_tag_ZN3K3b16GrowisofsHandlerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::GrowisofsHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b16GrowisofsHandlerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b16GrowisofsHandlerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b16GrowisofsHandlerE_t>.metaTypes,
    nullptr
} };

void K3b::GrowisofsHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GrowisofsHandler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->infoMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->newSubTask((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->buffer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->deviceBuffer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->flushingCache(); break;
        case 5: _t->reset((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->reset((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 7: _t->reset(); break;
        case 8: _t->handleStart(); break;
        case 9: _t->handleLine((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->handleExit((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->slotCheckBufferStatus(); break;
        case 12: _t->slotCheckBufferStatusDone((*reinterpret_cast<std::add_pointer_t<Device::DeviceHandler*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GrowisofsHandler::*)(const QString & , int )>(_a, &GrowisofsHandler::infoMessage, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GrowisofsHandler::*)(const QString & )>(_a, &GrowisofsHandler::newSubTask, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (GrowisofsHandler::*)(int )>(_a, &GrowisofsHandler::buffer, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (GrowisofsHandler::*)(int )>(_a, &GrowisofsHandler::deviceBuffer, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (GrowisofsHandler::*)()>(_a, &GrowisofsHandler::flushingCache, 4))
            return;
    }
}

const QMetaObject *K3b::GrowisofsHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::GrowisofsHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b16GrowisofsHandlerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int K3b::GrowisofsHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void K3b::GrowisofsHandler::infoMessage(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void K3b::GrowisofsHandler::newSubTask(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void K3b::GrowisofsHandler::buffer(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void K3b::GrowisofsHandler::deviceBuffer(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void K3b::GrowisofsHandler::flushingCache()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
