/****************************************************************************
** Meta object code from reading C++ file 'k3bmediaselectioncombobox.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/src/k3bmediaselectioncombobox.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3bmediaselectioncombobox.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b22MediaSelectionComboBoxE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::MediaSelectionComboBox::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b22MediaSelectionComboBoxE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::MediaSelectionComboBox",
        "selectionChanged",
        "",
        "K3b::Device::Device*",
        "newMedia",
        "newMedium",
        "dev",
        "setSelectedDevice",
        "setWantedMediumType",
        "K3b::Device::MediaTypes",
        "type",
        "setWantedMediumState",
        "K3b::Device::MediaStates",
        "state",
        "setWantedMediumContent",
        "K3b::Medium::MediumContents",
        "content",
        "setWantedMediumSize",
        "K3b::Msf",
        "minSize",
        "setIgnoreDevice",
        "slotMediumChanged",
        "slotDeviceManagerChanged",
        "K3b::Device::DeviceManager*",
        "slotActivated",
        "i",
        "slotUpdateToolTip"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selectionChanged'
        QtMocHelpers::SignalData<void(K3b::Device::Device *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'newMedia'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'newMedium'
        QtMocHelpers::SignalData<void(K3b::Device::Device *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 },
        }}),
        // Slot 'setSelectedDevice'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'setWantedMediumType'
        QtMocHelpers::SlotData<void(K3b::Device::MediaTypes)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'setWantedMediumState'
        QtMocHelpers::SlotData<void(K3b::Device::MediaStates)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'setWantedMediumContent'
        QtMocHelpers::SlotData<void(K3b::Medium::MediumContents)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'setWantedMediumSize'
        QtMocHelpers::SlotData<void(const K3b::Msf &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'setIgnoreDevice'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 },
        }}),
        // Slot 'slotMediumChanged'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'slotDeviceManagerChanged'
        QtMocHelpers::SlotData<void(K3b::Device::DeviceManager *)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 2 },
        }}),
        // Slot 'slotActivated'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'slotUpdateToolTip'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MediaSelectionComboBox, qt_meta_tag_ZN3K3b22MediaSelectionComboBoxE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::MediaSelectionComboBox::staticMetaObject = { {
    QMetaObject::SuperData::link<KComboBox::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b22MediaSelectionComboBoxE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b22MediaSelectionComboBoxE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b22MediaSelectionComboBoxE_t>.metaTypes,
    nullptr
} };

void K3b::MediaSelectionComboBox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MediaSelectionComboBox *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 1: _t->newMedia(); break;
        case 2: _t->newMedium((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 3: _t->setSelectedDevice((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 4: _t->setWantedMediumType((*reinterpret_cast<std::add_pointer_t<K3b::Device::MediaTypes>>(_a[1]))); break;
        case 5: _t->setWantedMediumState((*reinterpret_cast<std::add_pointer_t<K3b::Device::MediaStates>>(_a[1]))); break;
        case 6: _t->setWantedMediumContent((*reinterpret_cast<std::add_pointer_t<K3b::Medium::MediumContents>>(_a[1]))); break;
        case 7: _t->setWantedMediumSize((*reinterpret_cast<std::add_pointer_t<K3b::Msf>>(_a[1]))); break;
        case 8: _t->setIgnoreDevice((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 9: _t->slotMediumChanged((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 10: _t->slotDeviceManagerChanged((*reinterpret_cast<std::add_pointer_t<K3b::Device::DeviceManager*>>(_a[1]))); break;
        case 11: _t->slotActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->slotUpdateToolTip((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MediaSelectionComboBox::*)(K3b::Device::Device * )>(_a, &MediaSelectionComboBox::selectionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MediaSelectionComboBox::*)()>(_a, &MediaSelectionComboBox::newMedia, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MediaSelectionComboBox::*)(K3b::Device::Device * )>(_a, &MediaSelectionComboBox::newMedium, 2))
            return;
    }
}

const QMetaObject *K3b::MediaSelectionComboBox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::MediaSelectionComboBox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b22MediaSelectionComboBoxE_t>.strings))
        return static_cast<void*>(this);
    return KComboBox::qt_metacast(_clname);
}

int K3b::MediaSelectionComboBox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KComboBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void K3b::MediaSelectionComboBox::selectionChanged(K3b::Device::Device * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void K3b::MediaSelectionComboBox::newMedia()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void K3b::MediaSelectionComboBox::newMedium(K3b::Device::Device * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
