/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.k3b.AudioProject.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "k3baudioprojectinterfaceadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class K3bAudioProjectInterfaceAdaptor
 */

K3bAudioProjectInterfaceAdaptor::K3bAudioProjectInterfaceAdaptor(K3b::AudioProjectInterface *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

K3bAudioProjectInterfaceAdaptor::~K3bAudioProjectInterfaceAdaptor()
{
    // destructor
}

QString K3bAudioProjectInterfaceAdaptor::artist()
{
    // handle method call org.k3b.AudioProject.artist
    return parent()->artist();
}

void K3bAudioProjectInterfaceAdaptor::setArtist(const QString &artist)
{
    // handle method call org.k3b.AudioProject.setArtist
    parent()->setArtist(artist);
}

void K3bAudioProjectInterfaceAdaptor::setTitle(const QString &title)
{
    // handle method call org.k3b.AudioProject.setTitle
    parent()->setTitle(title);
}

void K3bAudioProjectInterfaceAdaptor::setTrackArtist(int trackNum, const QString &artist)
{
    // handle method call org.k3b.AudioProject.setTrackArtist
    parent()->setTrackArtist(trackNum, artist);
}

void K3bAudioProjectInterfaceAdaptor::setTrackTitle(int trackNum, const QString &title)
{
    // handle method call org.k3b.AudioProject.setTrackTitle
    parent()->setTrackTitle(trackNum, title);
}

QString K3bAudioProjectInterfaceAdaptor::title()
{
    // handle method call org.k3b.AudioProject.title
    return parent()->title();
}

QString K3bAudioProjectInterfaceAdaptor::trackArtist(int trackNum)
{
    // handle method call org.k3b.AudioProject.trackArtist
    return parent()->trackArtist(trackNum);
}

int K3bAudioProjectInterfaceAdaptor::trackCount()
{
    // handle method call org.k3b.AudioProject.trackCount
    return parent()->trackCount();
}

QString K3bAudioProjectInterfaceAdaptor::trackTitle(int trackNum)
{
    // handle method call org.k3b.AudioProject.trackTitle
    return parent()->trackTitle(trackNum);
}


#include "moc_k3baudioprojectinterfaceadaptor.cpp"
