/****************************************************************************
** Meta object code from reading C++ file 'k3bcdrecordwriter.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/libk3b/projects/k3bcdrecordwriter.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3bcdrecordwriter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b14CdrecordWriterE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::CdrecordWriter::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b14CdrecordWriterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::CdrecordWriter",
        "start",
        "",
        "cancel",
        "setDao",
        "b",
        "setWritingMode",
        "WritingMode",
        "mode",
        "setFormattingMode",
        "FormattingMode",
        "setCueFile",
        "s",
        "setClone",
        "setMulti",
        "setForce",
        "setRawCdText",
        "a",
        "slotStdLine",
        "line",
        "slotProcessExited",
        "exitCode",
        "QProcess::ExitStatus",
        "exitStatus",
        "slotThroughput",
        "t"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'start'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cancel'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setDao'
        QtMocHelpers::SlotData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'setWritingMode'
        QtMocHelpers::SlotData<void(WritingMode)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'setFormattingMode'
        QtMocHelpers::SlotData<void(FormattingMode)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 8 },
        }}),
        // Slot 'setCueFile'
        QtMocHelpers::SlotData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'setClone'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'setMulti'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'setForce'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'setRawCdText'
        QtMocHelpers::SlotData<void(const QByteArray &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 17 },
        }}),
        // Slot 'slotStdLine'
        QtMocHelpers::SlotData<void(const QString &)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'slotProcessExited'
        QtMocHelpers::SlotData<void(int, QProcess::ExitStatus)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 21 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotThroughput'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CdrecordWriter, qt_meta_tag_ZN3K3b14CdrecordWriterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::CdrecordWriter::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractWriter::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b14CdrecordWriterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b14CdrecordWriterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b14CdrecordWriterE_t>.metaTypes,
    nullptr
} };

void K3b::CdrecordWriter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CdrecordWriter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->start(); break;
        case 1: _t->cancel(); break;
        case 2: _t->setDao((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->setWritingMode((*reinterpret_cast<std::add_pointer_t<WritingMode>>(_a[1]))); break;
        case 4: _t->setFormattingMode((*reinterpret_cast<std::add_pointer_t<FormattingMode>>(_a[1]))); break;
        case 5: _t->setCueFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->setClone((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->setMulti((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->setForce((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->setRawCdText((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 10: _t->slotStdLine((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->slotProcessExited((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QProcess::ExitStatus>>(_a[2]))); break;
        case 12: _t->slotThroughput((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *K3b::CdrecordWriter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::CdrecordWriter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b14CdrecordWriterE_t>.strings))
        return static_cast<void*>(this);
    return AbstractWriter::qt_metacast(_clname);
}

int K3b::CdrecordWriter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractWriter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
