/****************************************************************************
** Meta object code from reading C++ file 'k3bqprocess.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/libk3b/tools/qprocess/k3bqprocess.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3bqprocess.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11K3bQProcessE_t {};
} // unnamed namespace

template <> constexpr inline auto K3bQProcess::qt_create_metaobjectdata<qt_meta_tag_ZN11K3bQProcessE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3bQProcess",
        "started",
        "",
        "finished",
        "exitCode",
        "QProcess::ExitStatus",
        "exitStatus",
        "error",
        "QProcess::ProcessError",
        "stateChanged",
        "QProcess::ProcessState",
        "state",
        "readyReadStandardOutput",
        "readyReadStandardError",
        "readyWrite",
        "terminate",
        "kill",
        "_q_canReadStandardOutput",
        "_q_canReadStandardError",
        "_q_canWrite",
        "_q_startupNotification",
        "_q_processDied",
        "_q_notifyProcessDied",
        "_q_notified"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'started'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(int, QProcess::ExitStatus)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'error'
        QtMocHelpers::SignalData<void(QProcess::ProcessError)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 7 },
        }}),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(QProcess::ProcessState)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'readyReadStandardOutput'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'readyReadStandardError'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'readyWrite'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'terminate'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'kill'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot '_q_canReadStandardOutput'
        QtMocHelpers::SlotData<bool()>(17, 2, QMC::AccessPrivate, QMetaType::Bool),
        // Slot '_q_canReadStandardError'
        QtMocHelpers::SlotData<bool()>(18, 2, QMC::AccessPrivate, QMetaType::Bool),
        // Slot '_q_canWrite'
        QtMocHelpers::SlotData<bool()>(19, 2, QMC::AccessPrivate, QMetaType::Bool),
        // Slot '_q_startupNotification'
        QtMocHelpers::SlotData<bool()>(20, 2, QMC::AccessPrivate, QMetaType::Bool),
        // Slot '_q_processDied'
        QtMocHelpers::SlotData<bool()>(21, 2, QMC::AccessPrivate, QMetaType::Bool),
        // Slot '_q_notifyProcessDied'
        QtMocHelpers::SlotData<bool()>(22, 2, QMC::AccessPrivate, QMetaType::Bool),
        // Slot '_q_notified'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<K3bQProcess, qt_meta_tag_ZN11K3bQProcessE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3bQProcess::staticMetaObject = { {
    QMetaObject::SuperData::link<QIODevice::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11K3bQProcessE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11K3bQProcessE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11K3bQProcessE_t>.metaTypes,
    nullptr
} };

void K3bQProcess::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<K3bQProcess *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->started(); break;
        case 1: _t->finished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->finished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QProcess::ExitStatus>>(_a[2]))); break;
        case 3: _t->error((*reinterpret_cast<std::add_pointer_t<QProcess::ProcessError>>(_a[1]))); break;
        case 4: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<QProcess::ProcessState>>(_a[1]))); break;
        case 5: _t->readyReadStandardOutput(); break;
        case 6: _t->readyReadStandardError(); break;
        case 7: _t->readyWrite(); break;
        case 8: _t->terminate(); break;
        case 9: _t->kill(); break;
        case 10: { bool _r = _t->d_func()->_q_canReadStandardOutput();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->d_func()->_q_canReadStandardError();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { bool _r = _t->d_func()->_q_canWrite();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->d_func()->_q_startupNotification();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->d_func()->_q_processDied();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: { bool _r = _t->d_func()->_q_notifyProcessDied();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->d_func()->_q_notified(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (K3bQProcess::*)()>(_a, &K3bQProcess::started, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bQProcess::*)(int )>(_a, &K3bQProcess::finished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bQProcess::*)(int , QProcess::ExitStatus )>(_a, &K3bQProcess::finished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bQProcess::*)(QProcess::ProcessError )>(_a, &K3bQProcess::error, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bQProcess::*)(QProcess::ProcessState )>(_a, &K3bQProcess::stateChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bQProcess::*)()>(_a, &K3bQProcess::readyReadStandardOutput, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bQProcess::*)()>(_a, &K3bQProcess::readyReadStandardError, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bQProcess::*)()>(_a, &K3bQProcess::readyWrite, 7))
            return;
    }
}

const QMetaObject *K3bQProcess::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3bQProcess::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11K3bQProcessE_t>.strings))
        return static_cast<void*>(this);
    return QIODevice::qt_metacast(_clname);
}

int K3bQProcess::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QIODevice::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void K3bQProcess::started()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void K3bQProcess::finished(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void K3bQProcess::finished(int _t1, QProcess::ExitStatus _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void K3bQProcess::error(QProcess::ProcessError _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void K3bQProcess::stateChanged(QProcess::ProcessState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void K3bQProcess::readyReadStandardOutput()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void K3bQProcess::readyReadStandardError()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void K3bQProcess::readyWrite()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
