/*
    SPDX-FileCopyrightText: 2010 Michal Malek <michalm@jabster.pl>
    SPDX-FileCopyrightText: 1998-2008 Sebastian Trueg <trueg@k3b.org>
    SPDX-License-Identifier: GPL-2.0-or-later
*/

#ifndef _K3B_SOX_ENCODER_DEFAULTS_H_
#define _K3B_SOX_ENCODER_DEFAULTS_H_

const bool DEFAULT_MANUAL_SETTINGS = false;
const int DEFAULT_CHANNELS = 2;
const int DEFAULT_SAMPLE_RATE = 44100;
const char DEFAULT_DATA_ENCODING[] = "signed";
const int DEFAULT_DATA_SIZE = 16;

#endif // _K3B_SOX_ENCODER_DEFAULTS_H_
