/****************************************************************************
** Meta object code from reading C++ file 'k3bdataprojectmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/src/projects/k3bdataprojectmodel.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3bdataprojectmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b16DataProjectModelE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::DataProjectModel::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b16DataProjectModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::DataProjectModel",
        "addUrlsRequested",
        "",
        "QList<QUrl>",
        "urls",
        "K3b::DirItem*",
        "targetDir",
        "moveItemsRequested",
        "QList<K3b::DataItem*>",
        "items",
        "_k_itemsAboutToBeInserted",
        "parent",
        "start",
        "end",
        "_k_itemsAboutToBeRemoved",
        "_k_itemsInserted",
        "_k_itemsRemoved",
        "_k_volumeIdChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'addUrlsRequested'
        QtMocHelpers::SignalData<void(QList<QUrl>, K3b::DirItem *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'moveItemsRequested'
        QtMocHelpers::SignalData<void(QList<K3b::DataItem*>, K3b::DirItem *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot '_k_itemsAboutToBeInserted'
        QtMocHelpers::SlotData<void(K3b::DirItem *, int, int)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 11 }, { QMetaType::Int, 12 }, { QMetaType::Int, 13 },
        }}),
        // Slot '_k_itemsAboutToBeRemoved'
        QtMocHelpers::SlotData<void(K3b::DirItem *, int, int)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 11 }, { QMetaType::Int, 12 }, { QMetaType::Int, 13 },
        }}),
        // Slot '_k_itemsInserted'
        QtMocHelpers::SlotData<void(K3b::DirItem *, int, int)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 11 }, { QMetaType::Int, 12 }, { QMetaType::Int, 13 },
        }}),
        // Slot '_k_itemsRemoved'
        QtMocHelpers::SlotData<void(K3b::DirItem *, int, int)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 11 }, { QMetaType::Int, 12 }, { QMetaType::Int, 13 },
        }}),
        // Slot '_k_volumeIdChanged'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DataProjectModel, qt_meta_tag_ZN3K3b16DataProjectModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::DataProjectModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b16DataProjectModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b16DataProjectModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b16DataProjectModelE_t>.metaTypes,
    nullptr
} };

void K3b::DataProjectModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DataProjectModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->addUrlsRequested((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<K3b::DirItem*>>(_a[2]))); break;
        case 1: _t->moveItemsRequested((*reinterpret_cast<std::add_pointer_t<QList<K3b::DataItem*>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<K3b::DirItem*>>(_a[2]))); break;
        case 2: _t->d->_k_itemsAboutToBeInserted((*reinterpret_cast<std::add_pointer_t<K3b::DirItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 3: _t->d->_k_itemsAboutToBeRemoved((*reinterpret_cast<std::add_pointer_t<K3b::DirItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 4: _t->d->_k_itemsInserted((*reinterpret_cast<std::add_pointer_t<K3b::DirItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 5: _t->d->_k_itemsRemoved((*reinterpret_cast<std::add_pointer_t<K3b::DirItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 6: _t->d->_k_volumeIdChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DataProjectModel::*)(QList<QUrl> , K3b::DirItem * )>(_a, &DataProjectModel::addUrlsRequested, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DataProjectModel::*)(QList<K3b::DataItem*> , K3b::DirItem * )>(_a, &DataProjectModel::moveItemsRequested, 1))
            return;
    }
}

const QMetaObject *K3b::DataProjectModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::DataProjectModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b16DataProjectModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int K3b::DataProjectModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void K3b::DataProjectModel::addUrlsRequested(QList<QUrl> _t1, K3b::DirItem * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void K3b::DataProjectModel::moveItemsRequested(QList<K3b::DataItem*> _t1, K3b::DirItem * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
