/****************************************************************************
** Meta object code from reading C++ file 'k3baudioprojectinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/src/k3baudioprojectinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3baudioprojectinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b21AudioProjectInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::AudioProjectInterface::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b21AudioProjectInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::AudioProjectInterface",
        "D-Bus Interface",
        "org.k3b.AudioProject",
        "trackCount",
        "",
        "title",
        "artist",
        "trackTitle",
        "trackNum",
        "trackArtist",
        "setTitle",
        "setArtist",
        "setTrackTitle",
        "setTrackArtist"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'trackCount'
        QtMocHelpers::SlotData<int() const>(3, 4, QMC::AccessPublic, QMetaType::Int),
        // Slot 'title'
        QtMocHelpers::SlotData<QString() const>(5, 4, QMC::AccessPublic, QMetaType::QString),
        // Slot 'artist'
        QtMocHelpers::SlotData<QString() const>(6, 4, QMC::AccessPublic, QMetaType::QString),
        // Slot 'trackTitle'
        QtMocHelpers::SlotData<QString(int) const>(7, 4, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'trackArtist'
        QtMocHelpers::SlotData<QString(int) const>(9, 4, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'setTitle'
        QtMocHelpers::SlotData<void(const QString &)>(10, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'setArtist'
        QtMocHelpers::SlotData<void(const QString &)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'setTrackTitle'
        QtMocHelpers::SlotData<void(int, const QString &)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::QString, 5 },
        }}),
        // Slot 'setTrackArtist'
        QtMocHelpers::SlotData<void(int, const QString &)>(13, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::QString, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<AudioProjectInterface, qt_meta_tag_ZN3K3b21AudioProjectInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject K3b::AudioProjectInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<ProjectInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b21AudioProjectInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b21AudioProjectInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b21AudioProjectInterfaceE_t>.metaTypes,
    nullptr
} };

void K3b::AudioProjectInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AudioProjectInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { int _r = _t->trackCount();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 1: { QString _r = _t->title();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { QString _r = _t->artist();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 3: { QString _r = _t->trackTitle((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 4: { QString _r = _t->trackArtist((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->setTitle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->setArtist((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->setTrackTitle((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->setTrackArtist((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *K3b::AudioProjectInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::AudioProjectInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b21AudioProjectInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return ProjectInterface::qt_metacast(_clname);
}

int K3b::AudioProjectInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ProjectInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
