/****************************************************************************
** Meta object code from reading C++ file 'k3binterfaceadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "k3binterfaceadaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3binterfaceadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19K3bInterfaceAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto K3bInterfaceAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN19K3bInterfaceAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3bInterfaceAdaptor",
        "D-Bus Interface",
        "org.k3b.MainWindow",
        "D-Bus Introspection",
        "  <interface name=\"org.k3b.MainWindow\">\n    <method name=\"crea"
        "teDataProject\">\n      <arg direction=\"out\" type=\"s\"/>\n    <"
        "/method>\n    <method name=\"createAudioProject\">\n      <arg dir"
        "ection=\"out\" type=\"s\"/>\n    </method>\n    <method name=\"cre"
        "ateMixedProject\">\n      <arg direction=\"out\" type=\"s\"/>\n   "
        " </method>\n    <method name=\"createVcdProject\">\n      <arg dir"
        "ection=\"out\" type=\"s\"/>\n    </method>\n    <method name=\"cre"
        "ateMovixProject\">\n      <arg direction=\"out\" type=\"s\"/>\n   "
        " </method>\n    <method name=\"createVideoDvdProject\">\n      <ar"
        "g direction=\"out\" type=\"s\"/>\n    </method>\n    <method name="
        "\"currentProject\">\n      <arg direction=\"out\" type=\"s\"/>\n  "
        "  </method>\n    <method name=\"openProject\">\n      <arg directi"
        "on=\"out\" type=\"s\"/>\n      <arg direction=\"in\" type=\"s\" na"
        "me=\"url\"/>\n    </method>\n    <method name=\"projects\">\n     "
        " <arg direction=\"out\" type=\"as\"/>\n    </method>\n    <method "
        "name=\"copyMedium\"/>\n    <method name=\"copyMedium\">\n      <ar"
        "g direction=\"in\" type=\"s\" name=\"dev\"/>\n    </method>\n    <"
        "method name=\"formatMedium\"/>\n    <method name=\"writeImage\"/>\n"
        "    <method name=\"writeImage\">\n      <arg direction=\"in\" type"
        "=\"s\" name=\"url\"/>\n    </method>\n    <method name=\"audioCdRi"
        "p\"/>\n    <method name=\"audioCdRip\">\n      <arg direction=\"in"
        "\" type=\"s\" name=\"dev\"/>\n    </method>\n    <method name=\"vi"
        "deoCdRip\"/>\n    <method name=\"videoCdRip\">\n      <arg directi"
        "on=\"in\" type=\"s\" name=\"dev\"/>\n    </method>\n    <method na"
        "me=\"videoDvdRip\"/>\n    <method name=\"videoDvdRip\">\n      <ar"
        "g direction=\"in\" type=\"s\" name=\"dev\"/>\n    </method>\n    <"
        "method name=\"addUrls\">\n      <arg direction=\"in\" type=\"as\" "
        "name=\"urls\"/>\n    </method>\n    <method name=\"addUrl\">\n    "
        "  <arg direction=\"in\" type=\"s\" name=\"url\"/>\n    </method>\n"
        "    <method name=\"blocked\">\n      <arg direction=\"out\" type=\""
        "b\"/>\n    </method>\n  </interface>\n",
        "addUrl",
        "",
        "url",
        "addUrls",
        "urls",
        "audioCdRip",
        "dev",
        "blocked",
        "copyMedium",
        "createAudioProject",
        "createDataProject",
        "createMixedProject",
        "createMovixProject",
        "createVcdProject",
        "createVideoDvdProject",
        "currentProject",
        "formatMedium",
        "openProject",
        "projects",
        "videoCdRip",
        "videoDvdRip",
        "writeImage"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'addUrl'
        QtMocHelpers::SlotData<void(const QString &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'addUrls'
        QtMocHelpers::SlotData<void(const QStringList &)>(8, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 9 },
        }}),
        // Slot 'audioCdRip'
        QtMocHelpers::SlotData<void(const QString &)>(10, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'audioCdRip'
        QtMocHelpers::SlotData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'blocked'
        QtMocHelpers::SlotData<bool()>(12, 6, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'copyMedium'
        QtMocHelpers::SlotData<void(const QString &)>(13, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'copyMedium'
        QtMocHelpers::SlotData<void()>(13, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'createAudioProject'
        QtMocHelpers::SlotData<QString()>(14, 6, QMC::AccessPublic, QMetaType::QString),
        // Slot 'createDataProject'
        QtMocHelpers::SlotData<QString()>(15, 6, QMC::AccessPublic, QMetaType::QString),
        // Slot 'createMixedProject'
        QtMocHelpers::SlotData<QString()>(16, 6, QMC::AccessPublic, QMetaType::QString),
        // Slot 'createMovixProject'
        QtMocHelpers::SlotData<QString()>(17, 6, QMC::AccessPublic, QMetaType::QString),
        // Slot 'createVcdProject'
        QtMocHelpers::SlotData<QString()>(18, 6, QMC::AccessPublic, QMetaType::QString),
        // Slot 'createVideoDvdProject'
        QtMocHelpers::SlotData<QString()>(19, 6, QMC::AccessPublic, QMetaType::QString),
        // Slot 'currentProject'
        QtMocHelpers::SlotData<QString()>(20, 6, QMC::AccessPublic, QMetaType::QString),
        // Slot 'formatMedium'
        QtMocHelpers::SlotData<void()>(21, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openProject'
        QtMocHelpers::SlotData<QString(const QString &)>(22, 6, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'projects'
        QtMocHelpers::SlotData<QStringList()>(23, 6, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'videoCdRip'
        QtMocHelpers::SlotData<void(const QString &)>(24, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'videoCdRip'
        QtMocHelpers::SlotData<void()>(24, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'videoDvdRip'
        QtMocHelpers::SlotData<void(const QString &)>(25, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'videoDvdRip'
        QtMocHelpers::SlotData<void()>(25, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'writeImage'
        QtMocHelpers::SlotData<void(const QString &)>(26, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'writeImage'
        QtMocHelpers::SlotData<void()>(26, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<K3bInterfaceAdaptor, qt_meta_tag_ZN19K3bInterfaceAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject K3bInterfaceAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19K3bInterfaceAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19K3bInterfaceAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19K3bInterfaceAdaptorE_t>.metaTypes,
    nullptr
} };

void K3bInterfaceAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<K3bInterfaceAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->addUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->addUrls((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 2: _t->audioCdRip((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->audioCdRip(); break;
        case 4: { bool _r = _t->blocked();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->copyMedium((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->copyMedium(); break;
        case 7: { QString _r = _t->createAudioProject();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 8: { QString _r = _t->createDataProject();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 9: { QString _r = _t->createMixedProject();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 10: { QString _r = _t->createMovixProject();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 11: { QString _r = _t->createVcdProject();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 12: { QString _r = _t->createVideoDvdProject();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 13: { QString _r = _t->currentProject();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->formatMedium(); break;
        case 15: { QString _r = _t->openProject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 16: { QStringList _r = _t->projects();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->videoCdRip((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->videoCdRip(); break;
        case 19: _t->videoDvdRip((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->videoDvdRip(); break;
        case 21: _t->writeImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->writeImage(); break;
        default: ;
        }
    }
}

const QMetaObject *K3bInterfaceAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3bInterfaceAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19K3bInterfaceAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int K3bInterfaceAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 23;
    }
    return _id;
}
QT_WARNING_POP
