/****************************************************************************
** Meta object code from reading C++ file 'k3baudioprojectinterfaceadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "k3baudioprojectinterfaceadaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3baudioprojectinterfaceadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN31K3bAudioProjectInterfaceAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto K3bAudioProjectInterfaceAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN31K3bAudioProjectInterfaceAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3bAudioProjectInterfaceAdaptor",
        "D-Bus Interface",
        "org.k3b.AudioProject",
        "D-Bus Introspection",
        "  <interface name=\"org.k3b.AudioProject\">\n    <method name=\"tr"
        "ackCount\">\n      <arg direction=\"out\" type=\"i\"/>\n    </meth"
        "od>\n    <method name=\"title\">\n      <arg direction=\"out\" typ"
        "e=\"s\"/>\n    </method>\n    <method name=\"artist\">\n      <arg"
        " direction=\"out\" type=\"s\"/>\n    </method>\n    <method name=\""
        "trackTitle\">\n      <arg direction=\"out\" type=\"s\"/>\n      <a"
        "rg direction=\"in\" type=\"i\" name=\"trackNum\"/>\n    </method>\n"
        "    <method name=\"trackArtist\">\n      <arg direction=\"out\" ty"
        "pe=\"s\"/>\n      <arg direction=\"in\" type=\"i\" name=\"trackNum"
        "\"/>\n    </method>\n    <method name=\"setTitle\">\n      <arg di"
        "rection=\"in\" type=\"s\" name=\"title\"/>\n    </method>\n    <me"
        "thod name=\"setArtist\">\n      <arg direction=\"in\" type=\"s\" n"
        "ame=\"artist\"/>\n    </method>\n    <method name=\"setTrackTitle\""
        ">\n      <arg direction=\"in\" type=\"i\" name=\"trackNum\"/>\n   "
        "   <arg direction=\"in\" type=\"s\" name=\"title\"/>\n    </method"
        ">\n    <method name=\"setTrackArtist\">\n      <arg direction=\"in"
        "\" type=\"i\" name=\"trackNum\"/>\n      <arg direction=\"in\" typ"
        "e=\"s\" name=\"artist\"/>\n    </method>\n  </interface>\n",
        "artist",
        "",
        "setArtist",
        "setTitle",
        "title",
        "setTrackArtist",
        "trackNum",
        "setTrackTitle",
        "trackArtist",
        "trackCount",
        "trackTitle"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'artist'
        QtMocHelpers::SlotData<QString()>(5, 6, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setArtist'
        QtMocHelpers::SlotData<void(const QString &)>(7, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'setTitle'
        QtMocHelpers::SlotData<void(const QString &)>(8, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Slot 'setTrackArtist'
        QtMocHelpers::SlotData<void(int, const QString &)>(10, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::QString, 5 },
        }}),
        // Slot 'setTrackTitle'
        QtMocHelpers::SlotData<void(int, const QString &)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::QString, 9 },
        }}),
        // Slot 'title'
        QtMocHelpers::SlotData<QString()>(9, 6, QMC::AccessPublic, QMetaType::QString),
        // Slot 'trackArtist'
        QtMocHelpers::SlotData<QString(int)>(13, 6, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'trackCount'
        QtMocHelpers::SlotData<int()>(14, 6, QMC::AccessPublic, QMetaType::Int),
        // Slot 'trackTitle'
        QtMocHelpers::SlotData<QString(int)>(15, 6, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Int, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<K3bAudioProjectInterfaceAdaptor, qt_meta_tag_ZN31K3bAudioProjectInterfaceAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject K3bAudioProjectInterfaceAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31K3bAudioProjectInterfaceAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31K3bAudioProjectInterfaceAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31K3bAudioProjectInterfaceAdaptorE_t>.metaTypes,
    nullptr
} };

void K3bAudioProjectInterfaceAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<K3bAudioProjectInterfaceAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QString _r = _t->artist();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->setArtist((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->setTitle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->setTrackArtist((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->setTrackTitle((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: { QString _r = _t->title();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 6: { QString _r = _t->trackArtist((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 7: { int _r = _t->trackCount();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 8: { QString _r = _t->trackTitle((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *K3bAudioProjectInterfaceAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3bAudioProjectInterfaceAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31K3bAudioProjectInterfaceAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int K3bAudioProjectInterfaceAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
