/********************************************************************************
** Form generated from reading UI file 'base_k3bexternalencodereditdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BASE_K3BEXTERNALENCODEREDITDIALOG_H
#define UI_BASE_K3BEXTERNALENCODEREDITDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_base_K3bExternalEncoderEditDialog
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox;
    QFormLayout *formLayout;
    QLabel *textLabel1;
    QLineEdit *m_editName;
    QLabel *textLabel3;
    QLineEdit *m_editExtension;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QLineEdit *m_editCommand;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *m_checkSwapByteOrder;
    QCheckBox *m_checkWriteWaveHeader;
    QDialogButtonBox *m_buttonBox;

    void setupUi(QDialog *base_K3bExternalEncoderEditDialog)
    {
        if (base_K3bExternalEncoderEditDialog->objectName().isEmpty())
            base_K3bExternalEncoderEditDialog->setObjectName("base_K3bExternalEncoderEditDialog");
        base_K3bExternalEncoderEditDialog->resize(533, 653);
        vboxLayout = new QVBoxLayout(base_K3bExternalEncoderEditDialog);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        groupBox = new QGroupBox(base_K3bExternalEncoderEditDialog);
        groupBox->setObjectName("groupBox");
        formLayout = new QFormLayout(groupBox);
        formLayout->setSpacing(6);
        formLayout->setContentsMargins(11, 11, 11, 11);
        formLayout->setObjectName("formLayout");
        textLabel1 = new QLabel(groupBox);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, textLabel1);

        m_editName = new QLineEdit(groupBox);
        m_editName->setObjectName("m_editName");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, m_editName);

        textLabel3 = new QLabel(groupBox);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, textLabel3);

        m_editExtension = new QLineEdit(groupBox);
        m_editExtension->setObjectName("m_editExtension");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, m_editExtension);

        m_editName->raise();
        textLabel1->raise();
        textLabel3->raise();
        m_editExtension->raise();

        vboxLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(base_K3bExternalEncoderEditDialog);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout = new QVBoxLayout(groupBox_2);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(groupBox_2);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        m_editCommand = new QLineEdit(groupBox_2);
        m_editCommand->setObjectName("m_editCommand");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_editCommand->sizePolicy().hasHeightForWidth());
        m_editCommand->setSizePolicy(sizePolicy1);

        verticalLayout->addWidget(m_editCommand);


        vboxLayout->addWidget(groupBox_2);

        groupBox_3 = new QGroupBox(base_K3bExternalEncoderEditDialog);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_2 = new QVBoxLayout(groupBox_3);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        m_checkSwapByteOrder = new QCheckBox(groupBox_3);
        m_checkSwapByteOrder->setObjectName("m_checkSwapByteOrder");

        verticalLayout_2->addWidget(m_checkSwapByteOrder);

        m_checkWriteWaveHeader = new QCheckBox(groupBox_3);
        m_checkWriteWaveHeader->setObjectName("m_checkWriteWaveHeader");

        verticalLayout_2->addWidget(m_checkWriteWaveHeader);


        vboxLayout->addWidget(groupBox_3);

        m_buttonBox = new QDialogButtonBox(base_K3bExternalEncoderEditDialog);
        m_buttonBox->setObjectName("m_buttonBox");
        m_buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(m_buttonBox);


        retranslateUi(base_K3bExternalEncoderEditDialog);
        QObject::connect(m_buttonBox, &QDialogButtonBox::accepted, base_K3bExternalEncoderEditDialog, qOverload<>(&QDialog::accept));
        QObject::connect(m_buttonBox, &QDialogButtonBox::rejected, base_K3bExternalEncoderEditDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(base_K3bExternalEncoderEditDialog);
    } // setupUi

    void retranslateUi(QDialog *base_K3bExternalEncoderEditDialog)
    {
        groupBox->setTitle(tr2i18n("General", nullptr));
        textLabel1->setText(tr2i18n("Name:", nullptr));
        textLabel3->setText(tr2i18n("Filename extension:", nullptr));
        groupBox_2->setTitle(tr2i18n("Command", nullptr));
        label->setText(tr2i18n("Please insert the command used to encode the audio data. The command has to read raw little endian (see <em>Swap Byte Order</em>) 16-bit stereo audio frames from stdin.\n"
"<p>The following strings will be replaced by K3b:<br>\n"
"<b>%f</b> - The filename of the resulting file. This is where the command has to write its output to.<br>\n"
"<em>The following refer to metadata stored for example in the ID3 tag of an mp3 file (Be aware that these values might be empty).</em><br>\n"
"<b>%t</b> - Title<br>\n"
"<b>%a</b> - Artist<br>\n"
"<b>%c</b> - Comment<br>\n"
"<b>%n</b> - Track number<br>\n"
"<b>%m</b> - Album Title<br>\n"
"<b>%r</b> - Album Artist<br>\n"
"<b>%x</b> - Album comment<br>\n"
"<b>%y</b> - Release Year", nullptr));
        groupBox_3->setTitle(tr2i18n("Options", nullptr));
#if QT_CONFIG(tooltip)
        m_checkSwapByteOrder->setToolTip(tr2i18n("Swap the byte order of the input data", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_checkSwapByteOrder->setWhatsThis(tr2i18n("<p> If this option is checked K3b will swap the byte order of the input data. Thus, the command has to read big endian audio frames.\n"
"<p>If the resulting audio file sounds bad it is highly likely that the byte order is wrong and this option has to be checked.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_checkSwapByteOrder->setText(tr2i18n("Swap &Byte Order", nullptr));
#if QT_CONFIG(tooltip)
        m_checkWriteWaveHeader->setToolTip(tr2i18n("Create a wave header for the input data", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_checkWriteWaveHeader->setWhatsThis(tr2i18n("<p>If this option is checked K3b will write a wave header. This is useful in case the encoder application cannot read plain raw audio data.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_checkWriteWaveHeader->setText(tr2i18n("Write W&ave Header", nullptr));
        (void)base_K3bExternalEncoderEditDialog;
    } // retranslateUi

};

namespace Ui {
    class base_K3bExternalEncoderEditDialog: public Ui_base_K3bExternalEncoderEditDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // BASE_K3BEXTERNALENCODEREDITDIALOG_H

