/********************************************************************************
** Form generated from reading UI file 'base_k3boggvorbisencodersettingswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BASE_K3BOGGVORBISENCODERSETTINGSWIDGET_H
#define UI_BASE_K3BOGGVORBISENCODERSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_base_K3bOggVorbisEncoderSettingsWidget
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox1;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout;
    QRadioButton *m_radioQualityLevel;
    QLabel *m_labelQualityLevel;
    QFrame *frame6;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacer2_2;
    QGridLayout *gridLayout;
    QSlider *m_slideQualityLevel;
    QLabel *textLabel4;
    QLabel *textLabel3_2;
    QSpacerItem *spacer9;
    QRadioButton *m_radioManual;
    QHBoxLayout *hboxLayout2;
    QSpacerItem *spacer2;
    QFrame *frame3;
    QGridLayout *gridLayout1;
    QCheckBox *m_checkBitrateUpper;
    QCheckBox *m_checkBitrateLower;
    QSpinBox *m_inputBitrateUpper;
    QSpinBox *m_inputBitrateNominal;
    QSpinBox *m_inputBitrateLower;
    QLabel *textLabel2;
    QLabel *textLabel3;
    QLabel *textLabel1;
    QCheckBox *m_checkBitrateNominal;

    void setupUi(QWidget *base_K3bOggVorbisEncoderSettingsWidget)
    {
        if (base_K3bOggVorbisEncoderSettingsWidget->objectName().isEmpty())
            base_K3bOggVorbisEncoderSettingsWidget->setObjectName("base_K3bOggVorbisEncoderSettingsWidget");
        base_K3bOggVorbisEncoderSettingsWidget->resize(335, 271);
        vboxLayout = new QVBoxLayout(base_K3bOggVorbisEncoderSettingsWidget);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName("vboxLayout");
        groupBox1 = new QGroupBox(base_K3bOggVorbisEncoderSettingsWidget);
        groupBox1->setObjectName("groupBox1");
        vboxLayout1 = new QVBoxLayout(groupBox1);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        m_radioQualityLevel = new QRadioButton(groupBox1);
        m_radioQualityLevel->setObjectName("m_radioQualityLevel");
        m_radioQualityLevel->setChecked(true);

        hboxLayout->addWidget(m_radioQualityLevel);

        m_labelQualityLevel = new QLabel(groupBox1);
        m_labelQualityLevel->setObjectName("m_labelQualityLevel");
        QFont font;
        font.setBold(true);
        m_labelQualityLevel->setFont(font);
        m_labelQualityLevel->setText(QString::fromUtf8("textLabel1"));
        m_labelQualityLevel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        m_labelQualityLevel->setWordWrap(false);

        hboxLayout->addWidget(m_labelQualityLevel);


        vboxLayout1->addLayout(hboxLayout);

        frame6 = new QFrame(groupBox1);
        frame6->setObjectName("frame6");
        frame6->setFrameShape(QFrame::NoFrame);
        frame6->setFrameShadow(QFrame::Plain);
        hboxLayout1 = new QHBoxLayout(frame6);
        hboxLayout1->setSpacing(6);
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName("hboxLayout1");
        spacer2_2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacer2_2);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        m_slideQualityLevel = new QSlider(frame6);
        m_slideQualityLevel->setObjectName("m_slideQualityLevel");
        m_slideQualityLevel->setMinimum(-1);
        m_slideQualityLevel->setMaximum(10);
        m_slideQualityLevel->setPageStep(3);
        m_slideQualityLevel->setValue(3);
        m_slideQualityLevel->setOrientation(Qt::Horizontal);

        gridLayout->addWidget(m_slideQualityLevel, 0, 0, 1, 2);

        textLabel4 = new QLabel(frame6);
        textLabel4->setObjectName("textLabel4");
        QFont font1;
        font1.setItalic(true);
        textLabel4->setFont(font1);
        textLabel4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel4->setWordWrap(false);

        gridLayout->addWidget(textLabel4, 1, 1, 1, 1);

        textLabel3_2 = new QLabel(frame6);
        textLabel3_2->setObjectName("textLabel3_2");
        textLabel3_2->setFont(font1);
        textLabel3_2->setWordWrap(false);

        gridLayout->addWidget(textLabel3_2, 1, 0, 1, 1);


        hboxLayout1->addLayout(gridLayout);


        vboxLayout1->addWidget(frame6);

        spacer9 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        vboxLayout1->addItem(spacer9);

        m_radioManual = new QRadioButton(groupBox1);
        m_radioManual->setObjectName("m_radioManual");

        vboxLayout1->addWidget(m_radioManual);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        spacer2 = new QSpacerItem(20, 21, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        hboxLayout2->addItem(spacer2);

        frame3 = new QFrame(groupBox1);
        frame3->setObjectName("frame3");
        frame3->setEnabled(false);
        frame3->setFrameShape(QFrame::NoFrame);
        frame3->setFrameShadow(QFrame::Raised);
        gridLayout1 = new QGridLayout(frame3);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        gridLayout1->setObjectName("gridLayout1");
        m_checkBitrateUpper = new QCheckBox(frame3);
        m_checkBitrateUpper->setObjectName("m_checkBitrateUpper");

        gridLayout1->addWidget(m_checkBitrateUpper, 0, 0, 1, 1);

        m_checkBitrateLower = new QCheckBox(frame3);
        m_checkBitrateLower->setObjectName("m_checkBitrateLower");

        gridLayout1->addWidget(m_checkBitrateLower, 2, 0, 1, 1);

        m_inputBitrateUpper = new QSpinBox(frame3);
        m_inputBitrateUpper->setObjectName("m_inputBitrateUpper");
        m_inputBitrateUpper->setEnabled(false);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_inputBitrateUpper->sizePolicy().hasHeightForWidth());
        m_inputBitrateUpper->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(m_inputBitrateUpper, 0, 1, 1, 1);

        m_inputBitrateNominal = new QSpinBox(frame3);
        m_inputBitrateNominal->setObjectName("m_inputBitrateNominal");
        m_inputBitrateNominal->setEnabled(false);
        sizePolicy.setHeightForWidth(m_inputBitrateNominal->sizePolicy().hasHeightForWidth());
        m_inputBitrateNominal->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(m_inputBitrateNominal, 1, 1, 1, 1);

        m_inputBitrateLower = new QSpinBox(frame3);
        m_inputBitrateLower->setObjectName("m_inputBitrateLower");
        m_inputBitrateLower->setEnabled(false);

        gridLayout1->addWidget(m_inputBitrateLower, 2, 1, 1, 1);

        textLabel2 = new QLabel(frame3);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout1->addWidget(textLabel2, 1, 2, 1, 1);

        textLabel3 = new QLabel(frame3);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout1->addWidget(textLabel3, 2, 2, 1, 1);

        textLabel1 = new QLabel(frame3);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout1->addWidget(textLabel1, 0, 2, 1, 1);

        m_checkBitrateNominal = new QCheckBox(frame3);
        m_checkBitrateNominal->setObjectName("m_checkBitrateNominal");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_checkBitrateNominal->sizePolicy().hasHeightForWidth());
        m_checkBitrateNominal->setSizePolicy(sizePolicy1);

        gridLayout1->addWidget(m_checkBitrateNominal, 1, 0, 1, 1);


        hboxLayout2->addWidget(frame3);


        vboxLayout1->addLayout(hboxLayout2);


        vboxLayout->addWidget(groupBox1);


        retranslateUi(base_K3bOggVorbisEncoderSettingsWidget);
        QObject::connect(m_radioManual, &QRadioButton::toggled, frame3, &QFrame::setEnabled);
        QObject::connect(m_checkBitrateUpper, &QCheckBox::toggled, m_inputBitrateUpper, &QSpinBox::setEnabled);
        QObject::connect(m_checkBitrateNominal, &QCheckBox::toggled, m_inputBitrateNominal, &QSpinBox::setEnabled);
        QObject::connect(m_checkBitrateLower, &QCheckBox::toggled, m_inputBitrateLower, &QSpinBox::setEnabled);
        QObject::connect(m_radioQualityLevel, &QRadioButton::toggled, m_labelQualityLevel, &QLabel::setEnabled);
        QObject::connect(m_radioQualityLevel, &QRadioButton::toggled, frame6, &QFrame::setEnabled);

        QMetaObject::connectSlotsByName(base_K3bOggVorbisEncoderSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *base_K3bOggVorbisEncoderSettingsWidget)
    {
        groupBox1->setProperty("title", QVariant(tr2i18n("File Quality", nullptr)));
#if QT_CONFIG(tooltip)
        m_radioQualityLevel->setToolTip(tr2i18n("Controls the quality of the encoded files", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_radioQualityLevel->setWhatsThis(tr2i18n("<p>Vorbis' audio quality is not best measured in kilobits per second, but on a scale from -1 to 10 called \"quality\". <p>For now, quality -1 is roughly equivalent to 45kbps average, 5 is roughly 160kbps, and 10 gives about 400kbps. Most people seeking very-near-CD-quality audio encode at a quality of 5 or, for lossless stereo coupling, 6. The default setting is quality 3, which at approximately 110kbps gives a smaller filesize and significantly better fidelity than .mp3 compression at 128kbps. <p><em>This explanation was copied from the www.vorbis.com FAQ.</em>", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_radioQualityLevel->setText(tr2i18n("&Quality level:", nullptr));
        textLabel4->setText(tr2i18n("high quality", nullptr));
        textLabel3_2->setText(tr2i18n("small file", nullptr));
        m_radioManual->setText(tr2i18n("M&anual settings:", nullptr));
        m_checkBitrateUpper->setText(tr2i18n("&Upper bitrate:", nullptr));
        m_checkBitrateLower->setText(tr2i18n("Lower &bitrate:", nullptr));
        textLabel2->setText(tr2i18n("kbps", nullptr));
        textLabel3->setText(tr2i18n("kbps", nullptr));
        textLabel1->setText(tr2i18n("kbps", nullptr));
        m_checkBitrateNominal->setText(tr2i18n("&Nominal bitrate:", nullptr));
        (void)base_K3bOggVorbisEncoderSettingsWidget;
    } // retranslateUi

};

namespace Ui {
    class base_K3bOggVorbisEncoderSettingsWidget: public Ui_base_K3bOggVorbisEncoderSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // BASE_K3BOGGVORBISENCODERSETTINGSWIDGET_H

