/****************************************************************************
** Meta object code from reading C++ file 'k3bdataprojectinterfaceadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "k3bdataprojectinterfaceadaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3bdataprojectinterfaceadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN30K3bDataProjectInterfaceAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto K3bDataProjectInterfaceAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN30K3bDataProjectInterfaceAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3bDataProjectInterfaceAdaptor",
        "D-Bus Interface",
        "org.k3b.DataProject",
        "D-Bus Introspection",
        "  <interface name=\"org.k3b.DataProject\">\n    <method name=\"cre"
        "ateFolder\">\n      <arg direction=\"out\" type=\"b\"/>\n      <ar"
        "g direction=\"in\" type=\"s\" name=\"name\"/>\n    </method>\n    "
        "<method name=\"createFolder\">\n      <arg direction=\"out\" type="
        "\"b\"/>\n      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n "
        "     <arg direction=\"in\" type=\"s\" name=\"dir\"/>\n    </method"
        ">\n    <method name=\"addUrl\">\n      <arg direction=\"in\" type="
        "\"s\" name=\"url\"/>\n      <arg direction=\"in\" type=\"s\" name="
        "\"dir\"/>\n    </method>\n    <method name=\"addUrls\">\n      <ar"
        "g direction=\"in\" type=\"as\" name=\"urls\"/>\n      <arg directi"
        "on=\"in\" type=\"s\" name=\"dir\"/>\n    </method>\n    <method na"
        "me=\"removeItem\">\n      <arg direction=\"out\" type=\"b\"/>\n   "
        "   <arg direction=\"in\" type=\"s\" name=\"path\"/>\n    </method>"
        "\n    <method name=\"renameItem\">\n      <arg direction=\"out\" t"
        "ype=\"b\"/>\n      <arg direction=\"in\" type=\"s\" name=\"path\"/"
        ">\n      <arg direction=\"in\" type=\"s\" name=\"newName\"/>\n    "
        "</method>\n    <method name=\"setVolumeID\">\n      <arg direction"
        "=\"in\" type=\"s\" name=\"id\"/>\n    </method>\n    <method name="
        "\"isFolder\">\n      <arg direction=\"out\" type=\"b\"/>\n      <a"
        "rg direction=\"in\" type=\"s\" name=\"path\"/>\n    </method>\n   "
        " <method name=\"children\">\n      <arg direction=\"out\" type=\"a"
        "s\"/>\n      <arg direction=\"in\" type=\"s\" name=\"path\"/>\n   "
        " </method>\n  </interface>\n",
        "addUrl",
        "",
        "url",
        "dir",
        "addUrls",
        "urls",
        "children",
        "path",
        "createFolder",
        "name",
        "isFolder",
        "removeItem",
        "renameItem",
        "newName",
        "setVolumeID",
        "id"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'addUrl'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::QString, 8 },
        }}),
        // Slot 'addUrls'
        QtMocHelpers::SlotData<void(const QStringList &, const QString &)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 10 }, { QMetaType::QString, 8 },
        }}),
        // Slot 'children'
        QtMocHelpers::SlotData<QStringList(const QString &)>(11, 6, QMC::AccessPublic, QMetaType::QStringList, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'createFolder'
        QtMocHelpers::SlotData<bool(const QString &, const QString &)>(13, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 14 }, { QMetaType::QString, 8 },
        }}),
        // Slot 'createFolder'
        QtMocHelpers::SlotData<bool(const QString &)>(13, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'isFolder'
        QtMocHelpers::SlotData<bool(const QString &)>(15, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'removeItem'
        QtMocHelpers::SlotData<bool(const QString &)>(16, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'renameItem'
        QtMocHelpers::SlotData<bool(const QString &, const QString &)>(17, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 12 }, { QMetaType::QString, 18 },
        }}),
        // Slot 'setVolumeID'
        QtMocHelpers::SlotData<void(const QString &)>(19, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<K3bDataProjectInterfaceAdaptor, qt_meta_tag_ZN30K3bDataProjectInterfaceAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject K3bDataProjectInterfaceAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30K3bDataProjectInterfaceAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30K3bDataProjectInterfaceAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN30K3bDataProjectInterfaceAdaptorE_t>.metaTypes,
    nullptr
} };

void K3bDataProjectInterfaceAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<K3bDataProjectInterfaceAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->addUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->addUrls((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: { QStringList _r = _t->children((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->createFolder((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->createFolder((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->isFolder((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->removeItem((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->renameItem((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->setVolumeID((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *K3bDataProjectInterfaceAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3bDataProjectInterfaceAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30K3bDataProjectInterfaceAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int K3bDataProjectInterfaceAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
