/****************************************************************************
** Meta object code from reading C++ file 'k3baudiodoc.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/libk3b/projects/audiocd/k3baudiodoc.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3baudiodoc.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b8AudioDocE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::AudioDoc::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b8AudioDocE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::AudioDoc",
        "trackChanged",
        "",
        "K3b::AudioTrack*",
        "track",
        "trackAboutToBeAdded",
        "position",
        "trackAdded",
        "trackAboutToBeRemoved",
        "trackRemoved",
        "sourceAboutToBeAdded",
        "parent",
        "sourceAdded",
        "sourceAboutToBeRemoved",
        "sourceRemoved",
        "addUrls",
        "QList<QUrl>",
        "addTrack",
        "QUrl",
        "addTracks",
        "AudioTrack*",
        "addSources",
        "urls",
        "AudioDataSource*",
        "sourceAfter",
        "removeTrack",
        "moveTrack",
        "after",
        "setHideFirstTrack",
        "b",
        "setNormalize",
        "writeCdText",
        "setTitle",
        "v",
        "setArtist",
        "setPerformer",
        "setDisc_id",
        "setArranger",
        "setSongwriter",
        "setComposer",
        "setUpc_ean",
        "setCdTextMessage",
        "setAudioRippingParanoiaMode",
        "i",
        "setAudioRippingRetries",
        "r",
        "setAudioRippingIgnoreReadErrors",
        "slotTrackChanged",
        "slotTrackRemoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'trackChanged'
        QtMocHelpers::SignalData<void(K3b::AudioTrack *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'trackAboutToBeAdded'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'trackAdded'
        QtMocHelpers::SignalData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'trackAboutToBeRemoved'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'trackRemoved'
        QtMocHelpers::SignalData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'sourceAboutToBeAdded'
        QtMocHelpers::SignalData<void(K3b::AudioTrack *, int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 11 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'sourceAdded'
        QtMocHelpers::SignalData<void(K3b::AudioTrack *, int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 11 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'sourceAboutToBeRemoved'
        QtMocHelpers::SignalData<void(K3b::AudioTrack *, int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 11 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'sourceRemoved'
        QtMocHelpers::SignalData<void(K3b::AudioTrack *, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 11 }, { QMetaType::Int, 6 },
        }}),
        // Slot 'addUrls'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 2 },
        }}),
        // Slot 'addTrack'
        QtMocHelpers::SlotData<void(const QUrl &, int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 2 }, { QMetaType::Int, 2 },
        }}),
        // Slot 'addTracks'
        QtMocHelpers::SlotData<void(const QList<QUrl> &, int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 2 }, { QMetaType::Int, 2 },
        }}),
        // Slot 'addTrack'
        QtMocHelpers::SlotData<void(AudioTrack *, int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 4 }, { QMetaType::Int, 6 },
        }}),
        // Slot 'addTrack'
        QtMocHelpers::SlotData<void(AudioTrack *)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 20, 4 },
        }}),
        // Slot 'addSources'
        QtMocHelpers::SlotData<void(AudioTrack *, const QList<QUrl> &, AudioDataSource *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 11 }, { 0x80000000 | 16, 22 }, { 0x80000000 | 23, 24 },
        }}),
        // Slot 'addSources'
        QtMocHelpers::SlotData<void(AudioTrack *, const QList<QUrl> &)>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 20, 11 }, { 0x80000000 | 16, 22 },
        }}),
        // Slot 'removeTrack'
        QtMocHelpers::SlotData<void(AudioTrack *)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 2 },
        }}),
        // Slot 'moveTrack'
        QtMocHelpers::SlotData<void(AudioTrack *, AudioTrack *)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 4 }, { 0x80000000 | 20, 27 },
        }}),
        // Slot 'setHideFirstTrack'
        QtMocHelpers::SlotData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Slot 'setNormalize'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Slot 'writeCdText'
        QtMocHelpers::SlotData<void(bool)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Slot 'setTitle'
        QtMocHelpers::SlotData<void(const QString &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setArtist'
        QtMocHelpers::SlotData<void(const QString &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setPerformer'
        QtMocHelpers::SlotData<void(const QString &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDisc_id'
        QtMocHelpers::SlotData<void(const QString &)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setArranger'
        QtMocHelpers::SlotData<void(const QString &)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setSongwriter'
        QtMocHelpers::SlotData<void(const QString &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setComposer'
        QtMocHelpers::SlotData<void(const QString &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setUpc_ean'
        QtMocHelpers::SlotData<void(const QString &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setCdTextMessage'
        QtMocHelpers::SlotData<void(const QString &)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setAudioRippingParanoiaMode'
        QtMocHelpers::SlotData<void(int)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 43 },
        }}),
        // Slot 'setAudioRippingRetries'
        QtMocHelpers::SlotData<void(int)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 45 },
        }}),
        // Slot 'setAudioRippingIgnoreReadErrors'
        QtMocHelpers::SlotData<void(bool)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Slot 'slotTrackChanged'
        QtMocHelpers::SlotData<void(K3b::AudioTrack *)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotTrackRemoved'
        QtMocHelpers::SlotData<void(int)>(48, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AudioDoc, qt_meta_tag_ZN3K3b8AudioDocE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::AudioDoc::staticMetaObject = { {
    QMetaObject::SuperData::link<Doc::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b8AudioDocE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b8AudioDocE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b8AudioDocE_t>.metaTypes,
    nullptr
} };

void K3b::AudioDoc::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AudioDoc *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->trackChanged((*reinterpret_cast<std::add_pointer_t<K3b::AudioTrack*>>(_a[1]))); break;
        case 1: _t->trackAboutToBeAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->trackAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->trackAboutToBeRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->trackRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->sourceAboutToBeAdded((*reinterpret_cast<std::add_pointer_t<K3b::AudioTrack*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->sourceAdded((*reinterpret_cast<std::add_pointer_t<K3b::AudioTrack*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 7: _t->sourceAboutToBeRemoved((*reinterpret_cast<std::add_pointer_t<K3b::AudioTrack*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->sourceRemoved((*reinterpret_cast<std::add_pointer_t<K3b::AudioTrack*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 9: _t->addUrls((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 10: _t->addTrack((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 11: _t->addTracks((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 12: _t->addTrack((*reinterpret_cast<std::add_pointer_t<AudioTrack*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 13: _t->addTrack((*reinterpret_cast<std::add_pointer_t<AudioTrack*>>(_a[1]))); break;
        case 14: _t->addSources((*reinterpret_cast<std::add_pointer_t<AudioTrack*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<AudioDataSource*>>(_a[3]))); break;
        case 15: _t->addSources((*reinterpret_cast<std::add_pointer_t<AudioTrack*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[2]))); break;
        case 16: _t->removeTrack((*reinterpret_cast<std::add_pointer_t<AudioTrack*>>(_a[1]))); break;
        case 17: _t->moveTrack((*reinterpret_cast<std::add_pointer_t<AudioTrack*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AudioTrack*>>(_a[2]))); break;
        case 18: _t->setHideFirstTrack((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->setNormalize((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->writeCdText((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->setTitle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->setArtist((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->setPerformer((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->setDisc_id((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 25: _t->setArranger((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 26: _t->setSongwriter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 27: _t->setComposer((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 28: _t->setUpc_ean((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 29: _t->setCdTextMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 30: _t->setAudioRippingParanoiaMode((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 31: _t->setAudioRippingRetries((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 32: _t->setAudioRippingIgnoreReadErrors((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->slotTrackChanged((*reinterpret_cast<std::add_pointer_t<K3b::AudioTrack*>>(_a[1]))); break;
        case 34: _t->slotTrackRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AudioDoc::*)(K3b::AudioTrack * )>(_a, &AudioDoc::trackChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AudioDoc::*)(int )>(_a, &AudioDoc::trackAboutToBeAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AudioDoc::*)(int )>(_a, &AudioDoc::trackAdded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AudioDoc::*)(int )>(_a, &AudioDoc::trackAboutToBeRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AudioDoc::*)(int )>(_a, &AudioDoc::trackRemoved, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AudioDoc::*)(K3b::AudioTrack * , int )>(_a, &AudioDoc::sourceAboutToBeAdded, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AudioDoc::*)(K3b::AudioTrack * , int )>(_a, &AudioDoc::sourceAdded, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AudioDoc::*)(K3b::AudioTrack * , int )>(_a, &AudioDoc::sourceAboutToBeRemoved, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AudioDoc::*)(K3b::AudioTrack * , int )>(_a, &AudioDoc::sourceRemoved, 8))
            return;
    }
}

const QMetaObject *K3b::AudioDoc::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::AudioDoc::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b8AudioDocE_t>.strings))
        return static_cast<void*>(this);
    return Doc::qt_metacast(_clname);
}

int K3b::AudioDoc::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Doc::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 35)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 35;
    }
    return _id;
}

// SIGNAL 0
void K3b::AudioDoc::trackChanged(K3b::AudioTrack * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void K3b::AudioDoc::trackAboutToBeAdded(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void K3b::AudioDoc::trackAdded(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void K3b::AudioDoc::trackAboutToBeRemoved(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void K3b::AudioDoc::trackRemoved(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void K3b::AudioDoc::sourceAboutToBeAdded(K3b::AudioTrack * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void K3b::AudioDoc::sourceAdded(K3b::AudioTrack * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void K3b::AudioDoc::sourceAboutToBeRemoved(K3b::AudioTrack * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void K3b::AudioDoc::sourceRemoved(K3b::AudioTrack * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}
QT_WARNING_POP
