/****************************************************************************
** Meta object code from reading C++ file 'k3bjobinterfaceadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "k3bjobinterfaceadaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3bjobinterfaceadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22K3bJobInterfaceAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto K3bJobInterfaceAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN22K3bJobInterfaceAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3bJobInterfaceAdaptor",
        "D-Bus Interface",
        "org.k3b.Job",
        "D-Bus Introspection",
        "  <interface name=\"org.k3b.Job\">\n    <signal name=\"started\"/>"
        "\n    <signal name=\"canceled\"/>\n    <signal name=\"finished\">\n"
        "      <arg direction=\"out\" type=\"b\"/>\n    </signal>\n    <sig"
        "nal name=\"infoMessage\">\n      <arg direction=\"out\" type=\"s\""
        "/>\n      <arg direction=\"out\" type=\"i\"/>\n    </signal>\n    "
        "<signal name=\"progress\">\n      <arg direction=\"out\" type=\"i\""
        "/>\n    </signal>\n    <signal name=\"subProgress\">\n      <arg d"
        "irection=\"out\" type=\"i\"/>\n    </signal>\n    <signal name=\"n"
        "ewTask\">\n      <arg direction=\"out\" type=\"s\"/>\n    </signal"
        ">\n    <signal name=\"newSubTask\">\n      <arg direction=\"out\" "
        "type=\"s\"/>\n    </signal>\n    <signal name=\"buffer\">\n      <"
        "arg direction=\"out\" type=\"i\"/>\n    </signal>\n    <signal nam"
        "e=\"deviceBuffer\">\n      <arg direction=\"out\" type=\"i\"/>\n  "
        "  </signal>\n    <signal name=\"nextTrack\">\n      <arg direction"
        "=\"out\" type=\"i\" name=\"track\"/>\n      <arg direction=\"out\""
        " type=\"i\" name=\"numTracks\"/>\n    </signal>\n    <method name="
        "\"jobRunning\">\n      <arg direction=\"out\" type=\"b\"/>\n    </"
        "method>\n    <method name=\"jobDescription\">\n      <arg directio"
        "n=\"out\" type=\"s\"/>\n    </method>\n    <method name=\"jobDetai"
        "ls\">\n      <arg direction=\"out\" type=\"s\"/>\n    </method>\n "
        " </interface>\n",
        "buffer",
        "",
        "in0",
        "canceled",
        "deviceBuffer",
        "finished",
        "infoMessage",
        "in1",
        "newSubTask",
        "newTask",
        "nextTrack",
        "track",
        "numTracks",
        "progress",
        "started",
        "subProgress",
        "jobDescription",
        "jobDetails",
        "jobRunning"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'buffer'
        QtMocHelpers::SignalData<void(int)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'canceled'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'deviceBuffer'
        QtMocHelpers::SignalData<void(int)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(bool)>(10, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'infoMessage'
        QtMocHelpers::SignalData<void(const QString &, int)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Int, 12 },
        }}),
        // Signal 'newSubTask'
        QtMocHelpers::SignalData<void(const QString &)>(13, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'newTask'
        QtMocHelpers::SignalData<void(const QString &)>(14, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'nextTrack'
        QtMocHelpers::SignalData<void(int, int)>(15, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 }, { QMetaType::Int, 17 },
        }}),
        // Signal 'progress'
        QtMocHelpers::SignalData<void(int)>(18, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'started'
        QtMocHelpers::SignalData<void()>(19, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subProgress'
        QtMocHelpers::SignalData<void(int)>(20, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'jobDescription'
        QtMocHelpers::SlotData<QString()>(21, 6, QMC::AccessPublic, QMetaType::QString),
        // Slot 'jobDetails'
        QtMocHelpers::SlotData<QString()>(22, 6, QMC::AccessPublic, QMetaType::QString),
        // Slot 'jobRunning'
        QtMocHelpers::SlotData<bool()>(23, 6, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<K3bJobInterfaceAdaptor, qt_meta_tag_ZN22K3bJobInterfaceAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject K3bJobInterfaceAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22K3bJobInterfaceAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22K3bJobInterfaceAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22K3bJobInterfaceAdaptorE_t>.metaTypes,
    nullptr
} };

void K3bJobInterfaceAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<K3bJobInterfaceAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->buffer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->canceled(); break;
        case 2: _t->deviceBuffer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->finished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->infoMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->newSubTask((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->newTask((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->nextTrack((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->progress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->started(); break;
        case 10: _t->subProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: { QString _r = _t->jobDescription();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 12: { QString _r = _t->jobDetails();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->jobRunning();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (K3bJobInterfaceAdaptor::*)(int )>(_a, &K3bJobInterfaceAdaptor::buffer, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bJobInterfaceAdaptor::*)()>(_a, &K3bJobInterfaceAdaptor::canceled, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bJobInterfaceAdaptor::*)(int )>(_a, &K3bJobInterfaceAdaptor::deviceBuffer, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bJobInterfaceAdaptor::*)(bool )>(_a, &K3bJobInterfaceAdaptor::finished, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bJobInterfaceAdaptor::*)(const QString & , int )>(_a, &K3bJobInterfaceAdaptor::infoMessage, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bJobInterfaceAdaptor::*)(const QString & )>(_a, &K3bJobInterfaceAdaptor::newSubTask, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bJobInterfaceAdaptor::*)(const QString & )>(_a, &K3bJobInterfaceAdaptor::newTask, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bJobInterfaceAdaptor::*)(int , int )>(_a, &K3bJobInterfaceAdaptor::nextTrack, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bJobInterfaceAdaptor::*)(int )>(_a, &K3bJobInterfaceAdaptor::progress, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bJobInterfaceAdaptor::*)()>(_a, &K3bJobInterfaceAdaptor::started, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (K3bJobInterfaceAdaptor::*)(int )>(_a, &K3bJobInterfaceAdaptor::subProgress, 10))
            return;
    }
}

const QMetaObject *K3bJobInterfaceAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3bJobInterfaceAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22K3bJobInterfaceAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int K3bJobInterfaceAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void K3bJobInterfaceAdaptor::buffer(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void K3bJobInterfaceAdaptor::canceled()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void K3bJobInterfaceAdaptor::deviceBuffer(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void K3bJobInterfaceAdaptor::finished(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void K3bJobInterfaceAdaptor::infoMessage(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void K3bJobInterfaceAdaptor::newSubTask(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void K3bJobInterfaceAdaptor::newTask(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void K3bJobInterfaceAdaptor::nextTrack(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void K3bJobInterfaceAdaptor::progress(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void K3bJobInterfaceAdaptor::started()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void K3bJobInterfaceAdaptor::subProgress(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
