/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.k3b.DataProject.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef K3BDATAPROJECTINTERFACEADAPTOR_H
#define K3BDATAPROJECTINTERFACEADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "k3bdataprojectinterface.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.k3b.DataProject
 */
class K3bDataProjectInterfaceAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.k3b.DataProject")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.k3b.DataProject\">\n"
"    <method name=\"createFolder\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"    </method>\n"
"    <method name=\"createFolder\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"dir\"/>\n"
"    </method>\n"
"    <method name=\"addUrl\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"url\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"dir\"/>\n"
"    </method>\n"
"    <method name=\"addUrls\">\n"
"      <arg direction=\"in\" type=\"as\" name=\"urls\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"dir\"/>\n"
"    </method>\n"
"    <method name=\"removeItem\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"path\"/>\n"
"    </method>\n"
"    <method name=\"renameItem\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"path\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"newName\"/>\n"
"    </method>\n"
"    <method name=\"setVolumeID\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"id\"/>\n"
"    </method>\n"
"    <method name=\"isFolder\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"path\"/>\n"
"    </method>\n"
"    <method name=\"children\">\n"
"      <arg direction=\"out\" type=\"as\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"path\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    K3bDataProjectInterfaceAdaptor(K3b::DataProjectInterface *parent);
    ~K3bDataProjectInterfaceAdaptor() override;

    inline K3b::DataProjectInterface *parent() const
    { return static_cast<K3b::DataProjectInterface *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    void addUrl(const QString &url, const QString &dir);
    void addUrls(const QStringList &urls, const QString &dir);
    QStringList children(const QString &path);
    bool createFolder(const QString &name, const QString &dir);
    bool createFolder(const QString &name);
    bool isFolder(const QString &path);
    bool removeItem(const QString &path);
    bool renameItem(const QString &path, const QString &newName);
    void setVolumeID(const QString &id);
Q_SIGNALS: // SIGNALS
};

#endif
