/****************************************************************************
** Meta object code from reading C++ file 'k3bwriterselectionwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/src/k3bwriterselectionwidget.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3bwriterselectionwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b21WriterSelectionWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::WriterSelectionWidget::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b21WriterSelectionWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::WriterSelectionWidget",
        "writerChanged",
        "",
        "K3b::Device::Device*",
        "writingAppChanged",
        "K3b::WritingApp",
        "app",
        "newMedia",
        "newMedium",
        "dev",
        "setWriterDevice",
        "setSpeed",
        "setWritingApp",
        "setSupportedWritingApps",
        "K3b::WritingApps",
        "apps",
        "setForceAutoSpeed",
        "setWantedMediumType",
        "K3b::Device::MediaTypes",
        "type",
        "setWantedMediumState",
        "K3b::Device::MediaStates",
        "state",
        "setWantedMediumSize",
        "K3b::Msf",
        "minSize",
        "setOverrideDevice",
        "overrideString",
        "tooltip",
        "setIgnoreDevice",
        "slotRefreshWriterSpeeds",
        "slotRefreshWritingApps",
        "slotWritingAppSelected",
        "id",
        "slotConfigChanged",
        "KSharedConfig::Ptr",
        "c",
        "slotSpeedChanged",
        "index",
        "slotWriterChanged",
        "slotNewBurnMedium",
        "slotManualSpeed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'writerChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'writerChanged'
        QtMocHelpers::SignalData<void(K3b::Device::Device *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'writingAppChanged'
        QtMocHelpers::SignalData<void(K3b::WritingApp)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'newMedia'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'newMedium'
        QtMocHelpers::SignalData<void(K3b::Device::Device *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 9 },
        }}),
        // Slot 'setWriterDevice'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'setSpeed'
        QtMocHelpers::SlotData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'setWritingApp'
        QtMocHelpers::SlotData<void(K3b::WritingApp)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setSupportedWritingApps'
        QtMocHelpers::SlotData<void(K3b::WritingApps)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'setForceAutoSpeed'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setWantedMediumType'
        QtMocHelpers::SlotData<void(K3b::Device::MediaTypes)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'setWantedMediumState'
        QtMocHelpers::SlotData<void(K3b::Device::MediaStates)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'setWantedMediumSize'
        QtMocHelpers::SlotData<void(const K3b::Msf &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'setOverrideDevice'
        QtMocHelpers::SlotData<void(K3b::Device::Device *, const QString &, const QString &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 9 }, { QMetaType::QString, 27 }, { QMetaType::QString, 28 },
        }}),
        // Slot 'setOverrideDevice'
        QtMocHelpers::SlotData<void(K3b::Device::Device *, const QString &)>(26, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 9 }, { QMetaType::QString, 27 },
        }}),
        // Slot 'setOverrideDevice'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(26, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 9 },
        }}),
        // Slot 'setIgnoreDevice'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 9 },
        }}),
        // Slot 'slotRefreshWriterSpeeds'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRefreshWritingApps'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotWritingAppSelected'
        QtMocHelpers::SlotData<void(int)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 33 },
        }}),
        // Slot 'slotConfigChanged'
        QtMocHelpers::SlotData<void(KSharedConfig::Ptr)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 35, 36 },
        }}),
        // Slot 'slotSpeedChanged'
        QtMocHelpers::SlotData<void(int)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 38 },
        }}),
        // Slot 'slotWriterChanged'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotNewBurnMedium'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 9 },
        }}),
        // Slot 'slotManualSpeed'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WriterSelectionWidget, qt_meta_tag_ZN3K3b21WriterSelectionWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::WriterSelectionWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b21WriterSelectionWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b21WriterSelectionWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b21WriterSelectionWidgetE_t>.metaTypes,
    nullptr
} };

void K3b::WriterSelectionWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WriterSelectionWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->writerChanged(); break;
        case 1: _t->writerChanged((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 2: _t->writingAppChanged((*reinterpret_cast<std::add_pointer_t<K3b::WritingApp>>(_a[1]))); break;
        case 3: _t->newMedia(); break;
        case 4: _t->newMedium((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 5: _t->setWriterDevice((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 6: _t->setSpeed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->setWritingApp((*reinterpret_cast<std::add_pointer_t<K3b::WritingApp>>(_a[1]))); break;
        case 8: _t->setSupportedWritingApps((*reinterpret_cast<std::add_pointer_t<K3b::WritingApps>>(_a[1]))); break;
        case 9: _t->setForceAutoSpeed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->setWantedMediumType((*reinterpret_cast<std::add_pointer_t<K3b::Device::MediaTypes>>(_a[1]))); break;
        case 11: _t->setWantedMediumState((*reinterpret_cast<std::add_pointer_t<K3b::Device::MediaStates>>(_a[1]))); break;
        case 12: _t->setWantedMediumSize((*reinterpret_cast<std::add_pointer_t<K3b::Msf>>(_a[1]))); break;
        case 13: _t->setOverrideDevice((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 14: _t->setOverrideDevice((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 15: _t->setOverrideDevice((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 16: _t->setIgnoreDevice((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 17: _t->slotRefreshWriterSpeeds(); break;
        case 18: _t->slotRefreshWritingApps(); break;
        case 19: _t->slotWritingAppSelected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->slotConfigChanged((*reinterpret_cast<std::add_pointer_t<KSharedConfig::Ptr>>(_a[1]))); break;
        case 21: _t->slotSpeedChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 22: _t->slotWriterChanged(); break;
        case 23: _t->slotNewBurnMedium((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 24: _t->slotManualSpeed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WriterSelectionWidget::*)()>(_a, &WriterSelectionWidget::writerChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WriterSelectionWidget::*)(K3b::Device::Device * )>(_a, &WriterSelectionWidget::writerChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WriterSelectionWidget::*)(K3b::WritingApp )>(_a, &WriterSelectionWidget::writingAppChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WriterSelectionWidget::*)()>(_a, &WriterSelectionWidget::newMedia, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WriterSelectionWidget::*)(K3b::Device::Device * )>(_a, &WriterSelectionWidget::newMedium, 4))
            return;
    }
}

const QMetaObject *K3b::WriterSelectionWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::WriterSelectionWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b21WriterSelectionWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int K3b::WriterSelectionWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void K3b::WriterSelectionWidget::writerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void K3b::WriterSelectionWidget::writerChanged(K3b::Device::Device * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void K3b::WriterSelectionWidget::writingAppChanged(K3b::WritingApp _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void K3b::WriterSelectionWidget::newMedia()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void K3b::WriterSelectionWidget::newMedium(K3b::Device::Device * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
