/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.k3b.Job.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef K3BJOBINTERFACEADAPTOR_H
#define K3BJOBINTERFACEADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "k3bjobinterface.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.k3b.Job
 */
class K3bJobInterfaceAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.k3b.Job")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.k3b.Job\">\n"
"    <signal name=\"started\"/>\n"
"    <signal name=\"canceled\"/>\n"
"    <signal name=\"finished\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"    </signal>\n"
"    <signal name=\"infoMessage\">\n"
"      <arg direction=\"out\" type=\"s\"/>\n"
"      <arg direction=\"out\" type=\"i\"/>\n"
"    </signal>\n"
"    <signal name=\"progress\">\n"
"      <arg direction=\"out\" type=\"i\"/>\n"
"    </signal>\n"
"    <signal name=\"subProgress\">\n"
"      <arg direction=\"out\" type=\"i\"/>\n"
"    </signal>\n"
"    <signal name=\"newTask\">\n"
"      <arg direction=\"out\" type=\"s\"/>\n"
"    </signal>\n"
"    <signal name=\"newSubTask\">\n"
"      <arg direction=\"out\" type=\"s\"/>\n"
"    </signal>\n"
"    <signal name=\"buffer\">\n"
"      <arg direction=\"out\" type=\"i\"/>\n"
"    </signal>\n"
"    <signal name=\"deviceBuffer\">\n"
"      <arg direction=\"out\" type=\"i\"/>\n"
"    </signal>\n"
"    <signal name=\"nextTrack\">\n"
"      <arg direction=\"out\" type=\"i\" name=\"track\"/>\n"
"      <arg direction=\"out\" type=\"i\" name=\"numTracks\"/>\n"
"    </signal>\n"
"    <method name=\"jobRunning\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"    </method>\n"
"    <method name=\"jobDescription\">\n"
"      <arg direction=\"out\" type=\"s\"/>\n"
"    </method>\n"
"    <method name=\"jobDetails\">\n"
"      <arg direction=\"out\" type=\"s\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    K3bJobInterfaceAdaptor(K3b::JobInterface *parent);
    ~K3bJobInterfaceAdaptor() override;

    inline K3b::JobInterface *parent() const
    { return static_cast<K3b::JobInterface *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    QString jobDescription();
    QString jobDetails();
    bool jobRunning();
Q_SIGNALS: // SIGNALS
    void buffer(int in0);
    void canceled();
    void deviceBuffer(int in0);
    void finished(bool in0);
    void infoMessage(const QString &in0, int in1);
    void newSubTask(const QString &in0);
    void newTask(const QString &in0);
    void nextTrack(int track, int numTracks);
    void progress(int in0);
    void started();
    void subProgress(int in0);
};

#endif
