/*
    SPDX-FileCopyrightText: 1998-2007 Sebastian Trueg <trueg@k3b.org>
    SPDX-License-Identifier: GPL-2.0-or-later
*/


#include "k3bplugin.h"


K3b::Plugin::Plugin( QObject* parent )
    : QObject( parent )
{
}


K3b::Plugin::~Plugin()
{
}

#include "moc_k3bplugin.cpp"
