/****************************************************************************
** Meta object code from reading C++ file 'k3bprojectinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/src/k3bprojectinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3bprojectinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b16ProjectInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::ProjectInterface::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b16ProjectInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::ProjectInterface",
        "D-Bus Interface",
        "org.k3b.Project",
        "addUrls",
        "",
        "urls",
        "addUrl",
        "url",
        "burn",
        "directBurn",
        "setBurnDevice",
        "blockdevicename",
        "length",
        "size",
        "KIO::filesize_t",
        "imagePath",
        "projectType"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'addUrls'
        QtMocHelpers::SlotData<void(const QStringList &)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 5 },
        }}),
        // Slot 'addUrl'
        QtMocHelpers::SlotData<void(const QString &)>(6, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'burn'
        QtMocHelpers::SlotData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'directBurn'
        QtMocHelpers::SlotData<bool()>(9, 4, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setBurnDevice'
        QtMocHelpers::SlotData<void(const QString &)>(10, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'length'
        QtMocHelpers::SlotData<int() const>(12, 4, QMC::AccessPublic, QMetaType::Int),
        // Slot 'size'
        QtMocHelpers::SlotData<KIO::filesize_t() const>(13, 4, QMC::AccessPublic, 0x80000000 | 14),
        // Slot 'imagePath'
        QtMocHelpers::SlotData<void() const>(15, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'projectType'
        QtMocHelpers::SlotData<QString() const>(16, 4, QMC::AccessPublic, QMetaType::QString),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<ProjectInterface, qt_meta_tag_ZN3K3b16ProjectInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject K3b::ProjectInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b16ProjectInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b16ProjectInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b16ProjectInterfaceE_t>.metaTypes,
    nullptr
} };

void K3b::ProjectInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ProjectInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->addUrls((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 1: _t->addUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->burn(); break;
        case 3: { bool _r = _t->directBurn();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->setBurnDevice((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: { int _r = _t->length();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 6: { KIO::filesize_t _r = _t->size();
            if (_a[0]) *reinterpret_cast<KIO::filesize_t*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->imagePath(); break;
        case 8: { QString _r = _t->projectType();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *K3b::ProjectInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::ProjectInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b16ProjectInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int K3b::ProjectInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
