/********************************************************************************
** Form generated from reading UI file 'base_k3bsoxencoderconfigwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BASE_K3BSOXENCODERCONFIGWIDGET_H
#define UI_BASE_K3BSOXENCODERCONFIGWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_base_K3bSoxEncoderConfigWidget
{
public:
    QVBoxLayout *vboxLayout;
    QCheckBox *m_checkManual;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacer2;
    QGridLayout *gridLayout;
    QLabel *textLabel2;
    QComboBox *m_comboEncoding;
    QLineEdit *m_editSamplerate;
    QLabel *textLabel4;
    QLabel *textLabel3;
    QLabel *textLabel1;
    QComboBox *m_comboChannels;
    QComboBox *m_comboSize;
    QSpacerItem *spacer1;

    void setupUi(QWidget *Form1)
    {
        if (Form1->objectName().isEmpty())
            Form1->setObjectName("Form1");
        Form1->resize(419, 201);
        vboxLayout = new QVBoxLayout(Form1);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName("vboxLayout");
        m_checkManual = new QCheckBox(Form1);
        m_checkManual->setObjectName("m_checkManual");

        vboxLayout->addWidget(m_checkManual);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        spacer2 = new QSpacerItem(20, 10, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer2);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        textLabel2 = new QLabel(Form1);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 1, 0, 1, 1);

        m_comboEncoding = new QComboBox(Form1);
        m_comboEncoding->addItem(QString());
        m_comboEncoding->addItem(QString());
        m_comboEncoding->addItem(QString());
        m_comboEncoding->addItem(QString());
        m_comboEncoding->addItem(QString());
        m_comboEncoding->addItem(QString());
        m_comboEncoding->addItem(QString());
        m_comboEncoding->addItem(QString());
        m_comboEncoding->setObjectName("m_comboEncoding");
        m_comboEncoding->setEnabled(false);

        gridLayout->addWidget(m_comboEncoding, 2, 1, 1, 1);

        m_editSamplerate = new QLineEdit(Form1);
        m_editSamplerate->setObjectName("m_editSamplerate");
        m_editSamplerate->setEnabled(false);

        gridLayout->addWidget(m_editSamplerate, 1, 1, 1, 1);

        textLabel4 = new QLabel(Form1);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        gridLayout->addWidget(textLabel4, 3, 0, 1, 1);

        textLabel3 = new QLabel(Form1);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout->addWidget(textLabel3, 2, 0, 1, 1);

        textLabel1 = new QLabel(Form1);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);

        m_comboChannels = new QComboBox(Form1);
        m_comboChannels->addItem(QString());
        m_comboChannels->addItem(QString());
        m_comboChannels->addItem(QString());
        m_comboChannels->setObjectName("m_comboChannels");
        m_comboChannels->setEnabled(false);

        gridLayout->addWidget(m_comboChannels, 0, 1, 1, 1);

        m_comboSize = new QComboBox(Form1);
        m_comboSize->addItem(QString());
        m_comboSize->addItem(QString());
        m_comboSize->addItem(QString());
        m_comboSize->setObjectName("m_comboSize");
        m_comboSize->setEnabled(false);

        gridLayout->addWidget(m_comboSize, 3, 1, 1, 1);


        hboxLayout->addLayout(gridLayout);


        vboxLayout->addLayout(hboxLayout);

        spacer1 = new QSpacerItem(250, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer1);


        retranslateUi(Form1);
        QObject::connect(m_checkManual, &QCheckBox::toggled, m_comboChannels, &QComboBox::setEnabled);
        QObject::connect(m_checkManual, &QCheckBox::toggled, m_editSamplerate, &QLineEdit::setEnabled);
        QObject::connect(m_checkManual, &QCheckBox::toggled, m_comboEncoding, &QComboBox::setEnabled);
        QObject::connect(m_checkManual, &QCheckBox::toggled, m_comboSize, &QComboBox::setEnabled);

        QMetaObject::connectSlotsByName(Form1);
    } // setupUi

    void retranslateUi(QWidget *Form1)
    {
        m_checkManual->setText(tr2i18n("Manual settings (used for all file types)", nullptr));
        textLabel2->setText(tr2i18n("Sample rate:", nullptr));
        m_comboEncoding->setItemText(0, tr2i18n("Signed Linear", nullptr));
        m_comboEncoding->setItemText(1, tr2i18n("Unsigned Linear", nullptr));
        m_comboEncoding->setItemText(2, tr2i18n("u-law (logarithmic)", nullptr));
        m_comboEncoding->setItemText(3, tr2i18n("A-law (logarithmic)", nullptr));
        m_comboEncoding->setItemText(4, tr2i18n("ADPCM", nullptr));
        m_comboEncoding->setItemText(5, tr2i18n("IMA_ADPCM", nullptr));
        m_comboEncoding->setItemText(6, tr2i18n("GSM", nullptr));
        m_comboEncoding->setItemText(7, tr2i18n("Floating-Point", nullptr));

#if QT_CONFIG(whatsthis)
        m_comboEncoding->setProperty("whatsThis", QVariant(tr2i18n("<p>The sample data encoding is signed linear (2's complement), unsigned linear, u-law (logarithmic), A-law (logarithmic), ADPCM, IMA_ADPCM, GSM, or Floating-point.</p>\n"
"<p><b>U-law</b> (actually shorthand for mu-law) and <b>A-law</b> are the U.S. and international standards for logarithmic telephone sound compression. When uncompressed u-law has roughly the precision of 14-bit PCM audio and A-law has roughly the precision of 13-bit PCM audio. A-law and u-law data is sometimes encoded using a reversed bit-ordering (i.e. MSB becomes LSB).<br> <b>ADPCM </b> is a form of sound compression that has a good compromise between good sound quality and fast encoding/decoding time. It is used for telephone sound compression and places where full fidelity is not as important. When uncompressed it has roughly the precision of 16-bit PCM audio. Popular versions of ADPCM include G.726, MS ADPCM, and IMA ADPCM. It has different meanings in different file handlers. In .wav files it represents MS ADPCM files, in all others it"
                        " means G.726 ADPCM. <br> <b>IMA ADPCM</b> is a specific form of ADPCM compression, slightly simpler and slightly lower fidelity than Microsoft's flavor of ADPCM. IMA ADPCM is also called DVI ADPCM.<br> <b>GSM</b> is a standard used for telephone sound compression in European countries and is gaining popularity because of its good quality. It is usually CPU intensive to work with GSM audio data.</p> <p><em>Description based on the SoX manpage</em></p>", nullptr)));
#endif // QT_CONFIG(whatsthis)
        m_editSamplerate->setText(tr2i18n("14400", nullptr));
#if QT_CONFIG(whatsthis)
        m_editSamplerate->setProperty("whatsThis", QVariant(QString()));
#endif // QT_CONFIG(whatsthis)
        textLabel4->setText(tr2i18n("Data size:", nullptr));
        textLabel3->setText(tr2i18n("Data encoding:", nullptr));
        textLabel1->setText(tr2i18n("Channels:", nullptr));
        m_comboChannels->setItemText(0, tr2i18n("1 (mono)", nullptr));
        m_comboChannels->setItemText(1, tr2i18n("2 (stereo)", nullptr));
        m_comboChannels->setItemText(2, tr2i18n("4 (quad sound)", nullptr));

        m_comboSize->setItemText(0, tr2i18n("Bytes", nullptr));
        m_comboSize->setItemText(1, tr2i18n("16-bit Words", nullptr));
        m_comboSize->setItemText(2, tr2i18n("32-bit Words", nullptr));

        (void)Form1;
    } // retranslateUi

};

namespace Ui {
    class base_K3bSoxEncoderConfigWidget: public Ui_base_K3bSoxEncoderConfigWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // BASE_K3BSOXENCODERCONFIGWIDGET_H

