/****************************************************************************
** Meta object code from reading C++ file 'k3baudiocdview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/src/rip/k3baudiocdview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3baudiocdview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b11AudioCdViewE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::AudioCdView::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b11AudioCdViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::AudioCdView",
        "loadCdInfo",
        "",
        "queryCddb",
        "readCdText",
        "slotContextMenu",
        "QPoint",
        "slotContextMenuAboutToShow",
        "slotTrackSelectionChanged",
        "slotSaveCddbLocally",
        "slotEditTrackCddb",
        "slotEditAlbumCddb",
        "startRip",
        "slotCheck",
        "slotUncheck",
        "slotToggle",
        "slotShowDataPart",
        "slotCddbChanged",
        "K3b::Device::Device*",
        "dev"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'loadCdInfo'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'queryCddb'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'readCdText'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Slot 'slotContextMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTrackSelectionChanged'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSaveCddbLocally'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotEditTrackCddb'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotEditAlbumCddb'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'startRip'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCheck'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUncheck'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotToggle'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotShowDataPart'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCddbChanged'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AudioCdView, qt_meta_tag_ZN3K3b11AudioCdViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::AudioCdView::staticMetaObject = { {
    QMetaObject::SuperData::link<MediaContentsView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b11AudioCdViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b11AudioCdViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b11AudioCdViewE_t>.metaTypes,
    nullptr
} };

void K3b::AudioCdView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AudioCdView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->loadCdInfo(); break;
        case 1: _t->queryCddb(); break;
        case 2: _t->readCdText(); break;
        case 3: _t->slotContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 4: _t->slotContextMenuAboutToShow(); break;
        case 5: _t->slotTrackSelectionChanged(); break;
        case 6: _t->slotSaveCddbLocally(); break;
        case 7: _t->slotEditTrackCddb(); break;
        case 8: _t->slotEditAlbumCddb(); break;
        case 9: _t->startRip(); break;
        case 10: _t->slotCheck(); break;
        case 11: _t->slotUncheck(); break;
        case 12: _t->slotToggle(); break;
        case 13: _t->slotShowDataPart(); break;
        case 14: _t->slotCddbChanged((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *K3b::AudioCdView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::AudioCdView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b11AudioCdViewE_t>.strings))
        return static_cast<void*>(this);
    return MediaContentsView::qt_metacast(_clname);
}

int K3b::AudioCdView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MediaContentsView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}
QT_WARNING_POP
