/********************************************************************************
** Form generated from reading UI file 'base_k3blameencodersettingswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BASE_K3BLAMEENCODERSETTINGSWIDGET_H
#define UI_BASE_K3BLAMEENCODERSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_K3bLameEncoderSettingsWidget
{
public:
    QVBoxLayout *verticalLayout_3;
    QTabWidget *m_mainTab;
    QWidget *tab1;
    QHBoxLayout *hboxLayout;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *hboxLayout1;
    QRadioButton *m_radioQualityLevel;
    QLabel *m_labelQualityLevel;
    QFrame *m_frameQualityLevel;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *spacerItem;
    QGridLayout *gridLayout;
    QLabel *textLabel4;
    QLabel *textLabel3;
    QSlider *m_sliderQuality;
    QSpacerItem *spacerItem1;
    QHBoxLayout *hboxLayout2;
    QRadioButton *m_radioManual;
    QLabel *m_labelManualSettings;
    QHBoxLayout *hboxLayout3;
    QSpacerItem *spacerItem2;
    QPushButton *m_buttonManualSettings;
    QSpacerItem *verticalSpacer;
    QWidget *tab2;
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_2;
    QSpinBox *m_spinEncoderQuality;
    QSlider *m_sliderEncoderQuality;
    QHBoxLayout *hboxLayout4;
    QLabel *textLabel1;
    QLabel *textLabel2;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QCheckBox *m_checkCopyright;
    QCheckBox *m_checkOriginal;
    QCheckBox *m_checkISO;
    QCheckBox *m_checkError;
    QSpacerItem *verticalSpacer_2;

    void setupUi(QWidget *K3bLameEncoderSettingsWidget)
    {
        if (K3bLameEncoderSettingsWidget->objectName().isEmpty())
            K3bLameEncoderSettingsWidget->setObjectName("K3bLameEncoderSettingsWidget");
        K3bLameEncoderSettingsWidget->resize(295, 347);
        verticalLayout_3 = new QVBoxLayout(K3bLameEncoderSettingsWidget);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        m_mainTab = new QTabWidget(K3bLameEncoderSettingsWidget);
        m_mainTab->setObjectName("m_mainTab");
        tab1 = new QWidget();
        tab1->setObjectName("tab1");
        hboxLayout = new QHBoxLayout(tab1);
        hboxLayout->setObjectName("hboxLayout");
        groupBox_3 = new QGroupBox(tab1);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_4 = new QVBoxLayout(groupBox_3);
        verticalLayout_4->setObjectName("verticalLayout_4");
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        m_radioQualityLevel = new QRadioButton(groupBox_3);
        m_radioQualityLevel->setObjectName("m_radioQualityLevel");
        m_radioQualityLevel->setChecked(true);

        hboxLayout1->addWidget(m_radioQualityLevel);

        m_labelQualityLevel = new QLabel(groupBox_3);
        m_labelQualityLevel->setObjectName("m_labelQualityLevel");
        QFont font;
        font.setBold(true);
        m_labelQualityLevel->setFont(font);
        m_labelQualityLevel->setText(QString::fromUtf8("textLabel1"));
        m_labelQualityLevel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        m_labelQualityLevel->setWordWrap(false);

        hboxLayout1->addWidget(m_labelQualityLevel);


        verticalLayout_4->addLayout(hboxLayout1);

        m_frameQualityLevel = new QFrame(groupBox_3);
        m_frameQualityLevel->setObjectName("m_frameQualityLevel");
        horizontalLayout = new QHBoxLayout(m_frameQualityLevel);
        horizontalLayout->setObjectName("horizontalLayout");
        spacerItem = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(spacerItem);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        textLabel4 = new QLabel(m_frameQualityLevel);
        textLabel4->setObjectName("textLabel4");
        QFont font1;
        font1.setItalic(true);
        textLabel4->setFont(font1);
        textLabel4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel4->setWordWrap(false);

        gridLayout->addWidget(textLabel4, 1, 1, 1, 1);

        textLabel3 = new QLabel(m_frameQualityLevel);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setFont(font1);
        textLabel3->setWordWrap(false);

        gridLayout->addWidget(textLabel3, 1, 0, 1, 1);

        m_sliderQuality = new QSlider(m_frameQualityLevel);
        m_sliderQuality->setObjectName("m_sliderQuality");
        m_sliderQuality->setMaximum(9);
        m_sliderQuality->setPageStep(3);
        m_sliderQuality->setValue(3);
        m_sliderQuality->setOrientation(Qt::Horizontal);

        gridLayout->addWidget(m_sliderQuality, 0, 0, 1, 2);


        horizontalLayout->addLayout(gridLayout);


        verticalLayout_4->addWidget(m_frameQualityLevel);

        spacerItem1 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout_4->addItem(spacerItem1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setObjectName("hboxLayout2");
        m_radioManual = new QRadioButton(groupBox_3);
        m_radioManual->setObjectName("m_radioManual");

        hboxLayout2->addWidget(m_radioManual);

        m_labelManualSettings = new QLabel(groupBox_3);
        m_labelManualSettings->setObjectName("m_labelManualSettings");
        m_labelManualSettings->setEnabled(false);
        m_labelManualSettings->setFont(font);
        m_labelManualSettings->setText(QString::fromUtf8("textLabel2"));
        m_labelManualSettings->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        m_labelManualSettings->setWordWrap(false);

        hboxLayout2->addWidget(m_labelManualSettings);


        verticalLayout_4->addLayout(hboxLayout2);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setSpacing(0);
        hboxLayout3->setObjectName("hboxLayout3");
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
        spacerItem2 = new QSpacerItem(20, 5, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout3->addItem(spacerItem2);

        m_buttonManualSettings = new QPushButton(groupBox_3);
        m_buttonManualSettings->setObjectName("m_buttonManualSettings");
        m_buttonManualSettings->setEnabled(false);

        hboxLayout3->addWidget(m_buttonManualSettings);


        verticalLayout_4->addLayout(hboxLayout3);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);


        hboxLayout->addWidget(groupBox_3);

        m_mainTab->addTab(tab1, QString());
        tab2 = new QWidget();
        tab2->setObjectName("tab2");
        vboxLayout = new QVBoxLayout(tab2);
        vboxLayout->setObjectName("vboxLayout");
        groupBox_2 = new QGroupBox(tab2);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_2 = new QVBoxLayout(groupBox_2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        m_spinEncoderQuality = new QSpinBox(groupBox_2);
        m_spinEncoderQuality->setObjectName("m_spinEncoderQuality");
        m_spinEncoderQuality->setMaximum(9);
        m_spinEncoderQuality->setValue(3);

        verticalLayout_2->addWidget(m_spinEncoderQuality);

        m_sliderEncoderQuality = new QSlider(groupBox_2);
        m_sliderEncoderQuality->setObjectName("m_sliderEncoderQuality");
        m_sliderEncoderQuality->setMaximum(9);
        m_sliderEncoderQuality->setPageStep(3);
        m_sliderEncoderQuality->setValue(3);
        m_sliderEncoderQuality->setOrientation(Qt::Horizontal);

        verticalLayout_2->addWidget(m_sliderEncoderQuality);

        hboxLayout4 = new QHBoxLayout();
        hboxLayout4->setObjectName("hboxLayout4");
        hboxLayout4->setContentsMargins(0, 0, 0, 0);
        textLabel1 = new QLabel(groupBox_2);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setFont(font1);
        textLabel1->setWordWrap(false);

        hboxLayout4->addWidget(textLabel1);

        textLabel2 = new QLabel(groupBox_2);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setFont(font1);
        textLabel2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel2->setWordWrap(false);

        hboxLayout4->addWidget(textLabel2);


        verticalLayout_2->addLayout(hboxLayout4);


        vboxLayout->addWidget(groupBox_2);

        groupBox = new QGroupBox(tab2);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName("verticalLayout");
        m_checkCopyright = new QCheckBox(groupBox);
        m_checkCopyright->setObjectName("m_checkCopyright");

        verticalLayout->addWidget(m_checkCopyright);

        m_checkOriginal = new QCheckBox(groupBox);
        m_checkOriginal->setObjectName("m_checkOriginal");

        verticalLayout->addWidget(m_checkOriginal);

        m_checkISO = new QCheckBox(groupBox);
        m_checkISO->setObjectName("m_checkISO");

        verticalLayout->addWidget(m_checkISO);

        m_checkError = new QCheckBox(groupBox);
        m_checkError->setObjectName("m_checkError");

        verticalLayout->addWidget(m_checkError);


        vboxLayout->addWidget(groupBox);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(verticalSpacer_2);

        m_mainTab->addTab(tab2, QString());

        verticalLayout_3->addWidget(m_mainTab);


        retranslateUi(K3bLameEncoderSettingsWidget);
        QObject::connect(m_radioManual, &QRadioButton::toggled, m_buttonManualSettings, &QPushButton::setEnabled);
        QObject::connect(m_radioQualityLevel, &QRadioButton::toggled, m_labelQualityLevel, &QLabel::setEnabled);
        QObject::connect(m_radioManual, &QRadioButton::toggled, m_labelManualSettings, &QLabel::setEnabled);
        QObject::connect(m_radioQualityLevel, &QRadioButton::toggled, m_frameQualityLevel, &QFrame::setEnabled);
        QObject::connect(m_sliderEncoderQuality, &QSlider::valueChanged, m_spinEncoderQuality, &QSpinBox::setValue);
        QObject::connect(m_spinEncoderQuality, &QSpinBox::valueChanged, m_sliderEncoderQuality, &QSlider::setValue);

        m_mainTab->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(K3bLameEncoderSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *K3bLameEncoderSettingsWidget)
    {
        groupBox_3->setTitle(tr2i18n("Quality Settings", nullptr));
        m_radioQualityLevel->setText(tr2i18n("Preset:", nullptr));
        textLabel4->setText(tr2i18n("high quality", nullptr));
        textLabel3->setText(tr2i18n("small file", nullptr));
        m_radioManual->setText(tr2i18n("Manua&l settings:", nullptr));
        m_buttonManualSettings->setText(tr2i18n("Change Settings...", nullptr));
        m_mainTab->setTabText(m_mainTab->indexOf(tab1), tr2i18n("Settings", nullptr));
        groupBox_2->setTitle(tr2i18n("Encoder Quality", nullptr));
#if QT_CONFIG(tooltip)
        m_spinEncoderQuality->setToolTip(tr2i18n("Choose the noise shaping & psycho acoustic algorithm.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_spinEncoderQuality->setWhatsThis(tr2i18n("<p>Bitrate is of course the main influence on quality. The higher the bitrate, the higher the quality. But for a given bitrate, we have a choice of algorithms to determine the best scalefactors and huffman encoding (noise shaping).\n"
"<p>The quality increases from 0 to 9 while the encoding speed drops.\n"
"<p>9 uses the slowest & best possible version of all algorithms.\n"
"<p><b>7 is the recommended setting</b> while 4 still produced reasonable quality at good speed.\n"
"<p>0 disables almost all algorithms including psy-model resulting in poor quality.\n"
"<p><b>This setting has no influence on the size of the resulting file.</b>", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1->setText(tr2i18n("fast encoding", nullptr));
        textLabel2->setText(tr2i18n("high quality", nullptr));
        groupBox->setTitle(tr2i18n("Options", nullptr));
#if QT_CONFIG(tooltip)
        m_checkCopyright->setToolTip(tr2i18n("Mark the encoded file as being copyrighted.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkCopyright->setText(tr2i18n("Mark copyrighted", nullptr));
#if QT_CONFIG(tooltip)
        m_checkOriginal->setToolTip(tr2i18n("Mark the encoded file as being a copy.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkOriginal->setText(tr2i18n("Mark as original", nullptr));
#if QT_CONFIG(tooltip)
        m_checkISO->setToolTip(tr2i18n("Enforce strict ISO compliance", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_checkISO->setWhatsThis(tr2i18n("<p>If this option is checked, LAME will enforce the 7680 bit limitation on total frame size.<br>\n"
"This results in many wasted bits for high bitrate encodings but will ensure strict ISO compatibility. This compatibility might be important for hardware players.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_checkISO->setText(tr2i18n("Strict ISO compliance", nullptr));
#if QT_CONFIG(tooltip)
        m_checkError->setToolTip(tr2i18n("Turn on CRC error protection.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_checkError->setWhatsThis(tr2i18n("<p>If this option is checked, a cyclic redundancy check (CRC) code will be added to each frame, allowing transmission errors that could occur on the MP3 stream to be detected; however, it takes 16 bits per frame that would otherwise be used for encoding, thus slightly reducing the sound quality.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_checkError->setText(tr2i18n("Error protection", nullptr));
        m_mainTab->setTabText(m_mainTab->indexOf(tab2), tr2i18n("Advanced", nullptr));
        (void)K3bLameEncoderSettingsWidget;
    } // retranslateUi

};

namespace Ui {
    class K3bLameEncoderSettingsWidget: public Ui_K3bLameEncoderSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // BASE_K3BLAMEENCODERSETTINGSWIDGET_H

