/****************************************************************************
** Meta object code from reading C++ file 'k3bjob.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/libk3b/core/k3bjob.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3bjob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b3JobE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::Job::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b3JobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::Job",
        "infoMessage",
        "",
        "msg",
        "type",
        "percent",
        "p",
        "subPercent",
        "processedSize",
        "processed",
        "size",
        "processedSubSize",
        "newTask",
        "job",
        "newSubTask",
        "debuggingOutput",
        "nextTrack",
        "track",
        "numTracks",
        "canceled",
        "started",
        "finished",
        "success",
        "start",
        "cancel",
        "setJobHandler",
        "K3b::JobHandler*",
        "slotCanceled",
        "slotNewSubTask",
        "str"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'infoMessage'
        QtMocHelpers::SignalData<void(const QString &, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 4 },
        }}),
        // Signal 'percent'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'subPercent'
        QtMocHelpers::SignalData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'processedSize'
        QtMocHelpers::SignalData<void(int, int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { QMetaType::Int, 10 },
        }}),
        // Signal 'processedSubSize'
        QtMocHelpers::SignalData<void(int, int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { QMetaType::Int, 10 },
        }}),
        // Signal 'newTask'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'newSubTask'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'debuggingOutput'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { QMetaType::QString, 2 },
        }}),
        // Signal 'nextTrack'
        QtMocHelpers::SignalData<void(int, int)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 }, { QMetaType::Int, 18 },
        }}),
        // Signal 'canceled'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'started'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'start'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cancel'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setJobHandler'
        QtMocHelpers::SlotData<void(K3b::JobHandler *)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 2 },
        }}),
        // Slot 'slotCanceled'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotNewSubTask'
        QtMocHelpers::SlotData<void(const QString &)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 29 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Job, qt_meta_tag_ZN3K3b3JobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::Job::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b3JobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b3JobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b3JobE_t>.metaTypes,
    nullptr
} };

void K3b::Job::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Job *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->infoMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->percent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->subPercent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->processedSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->processedSubSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->newTask((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->newSubTask((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->debuggingOutput((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->nextTrack((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 9: _t->canceled(); break;
        case 10: _t->started(); break;
        case 11: _t->finished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->start(); break;
        case 13: _t->cancel(); break;
        case 14: _t->setJobHandler((*reinterpret_cast<std::add_pointer_t<K3b::JobHandler*>>(_a[1]))); break;
        case 15: _t->slotCanceled(); break;
        case 16: _t->slotNewSubTask((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Job::*)(const QString & , int )>(_a, &Job::infoMessage, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Job::*)(int )>(_a, &Job::percent, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Job::*)(int )>(_a, &Job::subPercent, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Job::*)(int , int )>(_a, &Job::processedSize, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Job::*)(int , int )>(_a, &Job::processedSubSize, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Job::*)(const QString & )>(_a, &Job::newTask, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Job::*)(const QString & )>(_a, &Job::newSubTask, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Job::*)(const QString & , const QString & )>(_a, &Job::debuggingOutput, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Job::*)(int , int )>(_a, &Job::nextTrack, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Job::*)()>(_a, &Job::canceled, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Job::*)()>(_a, &Job::started, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Job::*)(bool )>(_a, &Job::finished, 11))
            return;
    }
}

const QMetaObject *K3b::Job::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::Job::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b3JobE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JobHandler"))
        return static_cast< JobHandler*>(this);
    return QObject::qt_metacast(_clname);
}

int K3b::Job::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void K3b::Job::infoMessage(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void K3b::Job::percent(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void K3b::Job::subPercent(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void K3b::Job::processedSize(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void K3b::Job::processedSubSize(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void K3b::Job::newTask(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void K3b::Job::newSubTask(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void K3b::Job::debuggingOutput(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void K3b::Job::nextTrack(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void K3b::Job::canceled()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void K3b::Job::started()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void K3b::Job::finished(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN3K3b7BurnJobE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::BurnJob::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b7BurnJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::BurnJob",
        "bufferStatus",
        "",
        "deviceBuffer",
        "writeSpeed",
        "speed",
        "K3b::Device::SpeedMultiplicator",
        "multiplicator",
        "burning",
        "setWritingApp",
        "K3b::WritingApp",
        "w"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'bufferStatus'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'deviceBuffer'
        QtMocHelpers::SignalData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'writeSpeed'
        QtMocHelpers::SignalData<void(int, K3b::Device::SpeedMultiplicator)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'burning'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setWritingApp'
        QtMocHelpers::SlotData<void(K3b::WritingApp)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BurnJob, qt_meta_tag_ZN3K3b7BurnJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::BurnJob::staticMetaObject = { {
    QMetaObject::SuperData::link<Job::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b7BurnJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b7BurnJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b7BurnJobE_t>.metaTypes,
    nullptr
} };

void K3b::BurnJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BurnJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->bufferStatus((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->deviceBuffer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->writeSpeed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<K3b::Device::SpeedMultiplicator>>(_a[2]))); break;
        case 3: _t->burning((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->setWritingApp((*reinterpret_cast<std::add_pointer_t<K3b::WritingApp>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (BurnJob::*)(int )>(_a, &BurnJob::bufferStatus, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (BurnJob::*)(int )>(_a, &BurnJob::deviceBuffer, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (BurnJob::*)(int , K3b::Device::SpeedMultiplicator )>(_a, &BurnJob::writeSpeed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (BurnJob::*)(bool )>(_a, &BurnJob::burning, 3))
            return;
    }
}

const QMetaObject *K3b::BurnJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::BurnJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b7BurnJobE_t>.strings))
        return static_cast<void*>(this);
    return Job::qt_metacast(_clname);
}

int K3b::BurnJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Job::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void K3b::BurnJob::bufferStatus(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void K3b::BurnJob::deviceBuffer(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void K3b::BurnJob::writeSpeed(int _t1, K3b::Device::SpeedMultiplicator _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void K3b::BurnJob::burning(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
