/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.k3b.AudioProject.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef K3BAUDIOPROJECTINTERFACEADAPTOR_H
#define K3BAUDIOPROJECTINTERFACEADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "k3baudioprojectinterface.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.k3b.AudioProject
 */
class K3bAudioProjectInterfaceAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.k3b.AudioProject")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.k3b.AudioProject\">\n"
"    <method name=\"trackCount\">\n"
"      <arg direction=\"out\" type=\"i\"/>\n"
"    </method>\n"
"    <method name=\"title\">\n"
"      <arg direction=\"out\" type=\"s\"/>\n"
"    </method>\n"
"    <method name=\"artist\">\n"
"      <arg direction=\"out\" type=\"s\"/>\n"
"    </method>\n"
"    <method name=\"trackTitle\">\n"
"      <arg direction=\"out\" type=\"s\"/>\n"
"      <arg direction=\"in\" type=\"i\" name=\"trackNum\"/>\n"
"    </method>\n"
"    <method name=\"trackArtist\">\n"
"      <arg direction=\"out\" type=\"s\"/>\n"
"      <arg direction=\"in\" type=\"i\" name=\"trackNum\"/>\n"
"    </method>\n"
"    <method name=\"setTitle\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"title\"/>\n"
"    </method>\n"
"    <method name=\"setArtist\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"artist\"/>\n"
"    </method>\n"
"    <method name=\"setTrackTitle\">\n"
"      <arg direction=\"in\" type=\"i\" name=\"trackNum\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"title\"/>\n"
"    </method>\n"
"    <method name=\"setTrackArtist\">\n"
"      <arg direction=\"in\" type=\"i\" name=\"trackNum\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"artist\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    K3bAudioProjectInterfaceAdaptor(K3b::AudioProjectInterface *parent);
    ~K3bAudioProjectInterfaceAdaptor() override;

    inline K3b::AudioProjectInterface *parent() const
    { return static_cast<K3b::AudioProjectInterface *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    QString artist();
    void setArtist(const QString &artist);
    void setTitle(const QString &title);
    void setTrackArtist(int trackNum, const QString &artist);
    void setTrackTitle(int trackNum, const QString &title);
    QString title();
    QString trackArtist(int trackNum);
    int trackCount();
    QString trackTitle(int trackNum);
Q_SIGNALS: // SIGNALS
};

#endif
