/****************************************************************************
** Meta object code from reading C++ file 'k3baudioeditorwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/src/projects/k3baudioeditorwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3baudioeditorwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b17AudioEditorWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::AudioEditorWidget::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b17AudioEditorWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::AudioEditorWidget",
        "mouseAt",
        "",
        "K3b::Msf",
        "rangeChanged",
        "identifier",
        "start",
        "end",
        "rangeRemoved",
        "selectedRangeChanged",
        "markerMoved",
        "pos",
        "markerAdded",
        "markerRemoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'mouseAt'
        QtMocHelpers::SignalData<void(const K3b::Msf &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'rangeChanged'
        QtMocHelpers::SignalData<void(int, const K3b::Msf &, const K3b::Msf &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { 0x80000000 | 3, 6 }, { 0x80000000 | 3, 7 },
        }}),
        // Signal 'rangeRemoved'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'selectedRangeChanged'
        QtMocHelpers::SignalData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'markerMoved'
        QtMocHelpers::SignalData<void(int, const K3b::Msf &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { 0x80000000 | 3, 11 },
        }}),
        // Signal 'markerAdded'
        QtMocHelpers::SignalData<void(int, const K3b::Msf &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { 0x80000000 | 3, 11 },
        }}),
        // Signal 'markerRemoved'
        QtMocHelpers::SignalData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AudioEditorWidget, qt_meta_tag_ZN3K3b17AudioEditorWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::AudioEditorWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b17AudioEditorWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b17AudioEditorWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b17AudioEditorWidgetE_t>.metaTypes,
    nullptr
} };

void K3b::AudioEditorWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AudioEditorWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->mouseAt((*reinterpret_cast<std::add_pointer_t<K3b::Msf>>(_a[1]))); break;
        case 1: _t->rangeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<K3b::Msf>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<K3b::Msf>>(_a[3]))); break;
        case 2: _t->rangeRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->selectedRangeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->markerMoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<K3b::Msf>>(_a[2]))); break;
        case 5: _t->markerAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<K3b::Msf>>(_a[2]))); break;
        case 6: _t->markerRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AudioEditorWidget::*)(const K3b::Msf & )>(_a, &AudioEditorWidget::mouseAt, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AudioEditorWidget::*)(int , const K3b::Msf & , const K3b::Msf & )>(_a, &AudioEditorWidget::rangeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AudioEditorWidget::*)(int )>(_a, &AudioEditorWidget::rangeRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AudioEditorWidget::*)(int )>(_a, &AudioEditorWidget::selectedRangeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AudioEditorWidget::*)(int , const K3b::Msf & )>(_a, &AudioEditorWidget::markerMoved, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AudioEditorWidget::*)(int , const K3b::Msf & )>(_a, &AudioEditorWidget::markerAdded, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AudioEditorWidget::*)(int )>(_a, &AudioEditorWidget::markerRemoved, 6))
            return;
    }
}

const QMetaObject *K3b::AudioEditorWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::AudioEditorWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b17AudioEditorWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int K3b::AudioEditorWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void K3b::AudioEditorWidget::mouseAt(const K3b::Msf & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void K3b::AudioEditorWidget::rangeChanged(int _t1, const K3b::Msf & _t2, const K3b::Msf & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void K3b::AudioEditorWidget::rangeRemoved(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void K3b::AudioEditorWidget::selectedRangeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void K3b::AudioEditorWidget::markerMoved(int _t1, const K3b::Msf & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void K3b::AudioEditorWidget::markerAdded(int _t1, const K3b::Msf & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void K3b::AudioEditorWidget::markerRemoved(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
