/****************************************************************************
** Meta object code from reading C++ file 'k3bdatadoc.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/libk3b/projects/datacd/k3bdatadoc.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3bdatadoc.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b7DataDocE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::DataDoc::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b7DataDocE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::DataDoc",
        "itemsAboutToBeInserted",
        "",
        "K3b::DirItem*",
        "parent",
        "start",
        "end",
        "itemsAboutToBeRemoved",
        "itemsInserted",
        "itemsRemoved",
        "volumeIdChanged",
        "importedSessionChanged",
        "importedSession",
        "addUrls",
        "QList<QUrl>",
        "urls",
        "addUrlsToDir",
        "dir",
        "clearImportedSession",
        "setVolumeID"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'itemsAboutToBeInserted'
        QtMocHelpers::SignalData<void(K3b::DirItem *, int, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'itemsAboutToBeRemoved'
        QtMocHelpers::SignalData<void(K3b::DirItem *, int, int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'itemsInserted'
        QtMocHelpers::SignalData<void(K3b::DirItem *, int, int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'itemsRemoved'
        QtMocHelpers::SignalData<void(K3b::DirItem *, int, int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'volumeIdChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'importedSessionChanged'
        QtMocHelpers::SignalData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'addUrls'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'addUrlsToDir'
        QtMocHelpers::SlotData<void(const QList<QUrl> &, K3b::DirItem *)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 3, 17 },
        }}),
        // Slot 'clearImportedSession'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setVolumeID'
        QtMocHelpers::SlotData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DataDoc, qt_meta_tag_ZN3K3b7DataDocE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::DataDoc::staticMetaObject = { {
    QMetaObject::SuperData::link<Doc::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b7DataDocE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b7DataDocE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b7DataDocE_t>.metaTypes,
    nullptr
} };

void K3b::DataDoc::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DataDoc *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->itemsAboutToBeInserted((*reinterpret_cast<std::add_pointer_t<K3b::DirItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 1: _t->itemsAboutToBeRemoved((*reinterpret_cast<std::add_pointer_t<K3b::DirItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 2: _t->itemsInserted((*reinterpret_cast<std::add_pointer_t<K3b::DirItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 3: _t->itemsRemoved((*reinterpret_cast<std::add_pointer_t<K3b::DirItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 4: _t->volumeIdChanged(); break;
        case 5: _t->importedSessionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->addUrls((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 7: _t->addUrlsToDir((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<K3b::DirItem*>>(_a[2]))); break;
        case 8: _t->clearImportedSession(); break;
        case 9: _t->setVolumeID((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DataDoc::*)(K3b::DirItem * , int , int )>(_a, &DataDoc::itemsAboutToBeInserted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DataDoc::*)(K3b::DirItem * , int , int )>(_a, &DataDoc::itemsAboutToBeRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DataDoc::*)(K3b::DirItem * , int , int )>(_a, &DataDoc::itemsInserted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DataDoc::*)(K3b::DirItem * , int , int )>(_a, &DataDoc::itemsRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DataDoc::*)()>(_a, &DataDoc::volumeIdChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DataDoc::*)(int )>(_a, &DataDoc::importedSessionChanged, 5))
            return;
    }
}

const QMetaObject *K3b::DataDoc::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::DataDoc::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b7DataDocE_t>.strings))
        return static_cast<void*>(this);
    return Doc::qt_metacast(_clname);
}

int K3b::DataDoc::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Doc::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void K3b::DataDoc::itemsAboutToBeInserted(K3b::DirItem * _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void K3b::DataDoc::itemsAboutToBeRemoved(K3b::DirItem * _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void K3b::DataDoc::itemsInserted(K3b::DirItem * _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void K3b::DataDoc::itemsRemoved(K3b::DirItem * _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void K3b::DataDoc::volumeIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void K3b::DataDoc::importedSessionChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
