/********************************************************************************
** Form generated from reading UI file 'base_k3bmanualbitratesettingsdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BASE_K3BMANUALBITRATESETTINGSDIALOG_H
#define UI_BASE_K3BMANUALBITRATESETTINGSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_K3bManualBitrateSettingsDialog
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QRadioButton *m_radioConstantBitrate;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QComboBox *m_comboConstantBitrate;
    QRadioButton *m_radioVariableBitrate;
    QFrame *m_frameVariableBitrate;
    QGridLayout *gridLayout_2;
    QCheckBox *m_checkBitrateMaximum;
    QComboBox *m_comboMaximumBitrate;
    QCheckBox *m_checkBitrateAverage;
    QSpinBox *m_spinAverageBitrate;
    QCheckBox *m_checkBitrateMinimum;
    QComboBox *m_comboMinimumBitrate;
    QSpacerItem *spacerItem1;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_2;
    QComboBox *m_comboMode;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *m_buttonBox;

    void setupUi(QDialog *K3bManualBitrateSettingsDialog)
    {
        if (K3bManualBitrateSettingsDialog->objectName().isEmpty())
            K3bManualBitrateSettingsDialog->setObjectName("K3bManualBitrateSettingsDialog");
        K3bManualBitrateSettingsDialog->resize(337, 388);
        vboxLayout = new QVBoxLayout(K3bManualBitrateSettingsDialog);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        groupBox = new QGroupBox(K3bManualBitrateSettingsDialog);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        m_radioConstantBitrate = new QRadioButton(groupBox);
        m_radioConstantBitrate->setObjectName("m_radioConstantBitrate");
        m_radioConstantBitrate->setChecked(true);

        verticalLayout->addWidget(m_radioConstantBitrate);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        spacerItem = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        m_comboConstantBitrate = new QComboBox(groupBox);
        m_comboConstantBitrate->setObjectName("m_comboConstantBitrate");

        hboxLayout->addWidget(m_comboConstantBitrate);


        verticalLayout->addLayout(hboxLayout);

        m_radioVariableBitrate = new QRadioButton(groupBox);
        m_radioVariableBitrate->setObjectName("m_radioVariableBitrate");

        verticalLayout->addWidget(m_radioVariableBitrate);

        m_frameVariableBitrate = new QFrame(groupBox);
        m_frameVariableBitrate->setObjectName("m_frameVariableBitrate");
        m_frameVariableBitrate->setEnabled(false);
        gridLayout_2 = new QGridLayout(m_frameVariableBitrate);
        gridLayout_2->setSpacing(6);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName("gridLayout_2");
        m_checkBitrateMaximum = new QCheckBox(m_frameVariableBitrate);
        m_checkBitrateMaximum->setObjectName("m_checkBitrateMaximum");

        gridLayout_2->addWidget(m_checkBitrateMaximum, 0, 1, 1, 1);

        m_comboMaximumBitrate = new QComboBox(m_frameVariableBitrate);
        m_comboMaximumBitrate->setObjectName("m_comboMaximumBitrate");
        m_comboMaximumBitrate->setEnabled(false);

        gridLayout_2->addWidget(m_comboMaximumBitrate, 0, 2, 1, 1);

        m_checkBitrateAverage = new QCheckBox(m_frameVariableBitrate);
        m_checkBitrateAverage->setObjectName("m_checkBitrateAverage");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_checkBitrateAverage->sizePolicy().hasHeightForWidth());
        m_checkBitrateAverage->setSizePolicy(sizePolicy);

        gridLayout_2->addWidget(m_checkBitrateAverage, 1, 1, 1, 1);

        m_spinAverageBitrate = new QSpinBox(m_frameVariableBitrate);
        m_spinAverageBitrate->setObjectName("m_spinAverageBitrate");
        m_spinAverageBitrate->setEnabled(false);
        m_spinAverageBitrate->setButtonSymbols(QAbstractSpinBox::UpDownArrows);
        m_spinAverageBitrate->setMinimum(8);
        m_spinAverageBitrate->setMaximum(310);

        gridLayout_2->addWidget(m_spinAverageBitrate, 1, 2, 1, 1);

        m_checkBitrateMinimum = new QCheckBox(m_frameVariableBitrate);
        m_checkBitrateMinimum->setObjectName("m_checkBitrateMinimum");

        gridLayout_2->addWidget(m_checkBitrateMinimum, 2, 1, 1, 1);

        m_comboMinimumBitrate = new QComboBox(m_frameVariableBitrate);
        m_comboMinimumBitrate->setObjectName("m_comboMinimumBitrate");
        m_comboMinimumBitrate->setEnabled(false);

        gridLayout_2->addWidget(m_comboMinimumBitrate, 2, 2, 1, 1);

        spacerItem1 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(spacerItem1, 1, 0, 1, 1);


        verticalLayout->addWidget(m_frameVariableBitrate);


        vboxLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(K3bManualBitrateSettingsDialog);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_2 = new QVBoxLayout(groupBox_2);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        m_comboMode = new QComboBox(groupBox_2);
        m_comboMode->addItem(QString());
        m_comboMode->addItem(QString());
        m_comboMode->addItem(QString());
        m_comboMode->setObjectName("m_comboMode");

        verticalLayout_2->addWidget(m_comboMode);


        vboxLayout->addWidget(groupBox_2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(verticalSpacer);

        m_buttonBox = new QDialogButtonBox(K3bManualBitrateSettingsDialog);
        m_buttonBox->setObjectName("m_buttonBox");
        m_buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(m_buttonBox);


        retranslateUi(K3bManualBitrateSettingsDialog);
        QObject::connect(m_radioConstantBitrate, &QRadioButton::toggled, m_comboConstantBitrate, &QComboBox::setEnabled);
        QObject::connect(m_checkBitrateMaximum, &QCheckBox::toggled, m_comboMaximumBitrate, &QComboBox::setEnabled);
        QObject::connect(m_checkBitrateAverage, &QCheckBox::toggled, m_spinAverageBitrate, &QSpinBox::setEnabled);
        QObject::connect(m_checkBitrateMinimum, &QCheckBox::toggled, m_comboMinimumBitrate, &QComboBox::setEnabled);
        QObject::connect(m_radioVariableBitrate, &QRadioButton::toggled, m_frameVariableBitrate, &QFrame::setEnabled);
        QObject::connect(m_buttonBox, &QDialogButtonBox::accepted, K3bManualBitrateSettingsDialog, qOverload<>(&QDialog::accept));
        QObject::connect(m_buttonBox, &QDialogButtonBox::rejected, K3bManualBitrateSettingsDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(K3bManualBitrateSettingsDialog);
    } // setupUi

    void retranslateUi(QDialog *K3bManualBitrateSettingsDialog)
    {
        groupBox->setTitle(tr2i18n("Quality", nullptr));
        m_radioConstantBitrate->setText(tr2i18n("&Constant Bitrate", nullptr));
        m_radioVariableBitrate->setText(tr2i18n("&Variable Bitrate", nullptr));
        m_checkBitrateMaximum->setText(tr2i18n("Maximum bitrate:", nullptr));
        m_checkBitrateAverage->setText(tr2i18n("Average bitrate:", nullptr));
        m_spinAverageBitrate->setSuffix(tr2i18n(" kbps", nullptr));
        m_checkBitrateMinimum->setText(tr2i18n("Minimum bitrate:", nullptr));
        groupBox_2->setTitle(tr2i18n("Channel Mode", nullptr));
        m_comboMode->setItemText(0, tr2i18n("Stereo", nullptr));
        m_comboMode->setItemText(1, tr2i18n("Joint Stereo", nullptr));
        m_comboMode->setItemText(2, tr2i18n("Mono", nullptr));

#if QT_CONFIG(tooltip)
        m_comboMode->setToolTip(tr2i18n("Select the channel mode.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_comboMode->setWhatsThis(tr2i18n("<p>Select the channel mode of the resulting MP3 file:\n"
"<p><b>Stereo</b><br>\n"
"In this mode, the encoder makes no use of potential correlations between the two input channels; it can, however, negotiate the bit demand between both channel, i.e. give one channel more bits if the other contains silence or needs fewer bits because of a lower complexity.\n"
"<p><b>Joint-Stereo</b><br>\n"
"In this mode, the encoder will make use of correlations between both channels. The signal will be matrixed into a sum (\"mid\"), computed by L+R, and difference (\"side\") signal, computed by L-R, and more bits are allocated to the mid channel. This will effectively increase the bandwidth if the signal does not have too much stereo separation, thus giving a significant gain in encoding quality.\n"
"<p><b>Mono</b><br>\n"
"The input will be encoded as a mono signal. If it was a stereo signal, it will be downsampled to mono. The downmix is calculated as the sum of the left and right channel, attenuated by 6 dB.", nullptr));
#endif // QT_CONFIG(whatsthis)
        (void)K3bManualBitrateSettingsDialog;
    } // retranslateUi

};

namespace Ui {
    class K3bManualBitrateSettingsDialog: public Ui_K3bManualBitrateSettingsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // BASE_K3BMANUALBITRATESETTINGSDIALOG_H

