/****************************************************************************
** Meta object code from reading C++ file 'k3b.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/src/k3b.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3b.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::MainWindow",
        "initializationInfo",
        "",
        "configChanged",
        "KSharedConfig::Ptr",
        "c",
        "slotNewAudioDoc",
        "K3b::Doc*",
        "slotNewDataDoc",
        "slotNewMixedDoc",
        "slotNewVcdDoc",
        "slotNewMovixDoc",
        "slotNewVideoDvdDoc",
        "slotContinueMultisession",
        "slotClearProject",
        "slotWriteImage",
        "QUrl",
        "url",
        "formatMedium",
        "K3b::Device::Device*",
        "slotFormatMedium",
        "mediaCopy",
        "slotMediaCopy",
        "cddaRip",
        "slotCddaRip",
        "videoDvdRip",
        "slotVideoDvdRip",
        "videoCdRip",
        "slotVideoCdRip",
        "showDiskInfo",
        "slotErrorMessage",
        "slotWarningMessage",
        "slotConfigureKeys",
        "slotCheckSystem",
        "slotManualCheckSystem",
        "addUrls",
        "QList<QUrl>",
        "urls",
        "slotFileOpen",
        "slotFileOpenRecent",
        "slotFileSave",
        "slotFileSaveAs",
        "slotFileSaveAll",
        "slotFileClose",
        "Doc*",
        "doc",
        "slotFileCloseAll",
        "slotSettingsConfigure",
        "slotCurrentDocChanged",
        "slotFileQuit",
        "slotViewStatusBar",
        "slotStatusMsg",
        "text",
        "slotShowMenuBar",
        "slotProjectAddFiles",
        "slotEditToolbars",
        "slotNewToolBarConfig",
        "slotDataImportSession",
        "slotDataClearImportedSession",
        "slotEditBootImages",
        "createClient",
        "slotCheckSystemTimed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'initializationInfo'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'configChanged'
        QtMocHelpers::SignalData<void(KSharedConfig::Ptr)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotNewAudioDoc'
        QtMocHelpers::SlotData<K3b::Doc *()>(6, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'slotNewDataDoc'
        QtMocHelpers::SlotData<K3b::Doc *()>(8, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'slotNewMixedDoc'
        QtMocHelpers::SlotData<K3b::Doc *()>(9, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'slotNewVcdDoc'
        QtMocHelpers::SlotData<K3b::Doc *()>(10, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'slotNewMovixDoc'
        QtMocHelpers::SlotData<K3b::Doc *()>(11, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'slotNewVideoDvdDoc'
        QtMocHelpers::SlotData<K3b::Doc *()>(12, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'slotContinueMultisession'
        QtMocHelpers::SlotData<K3b::Doc *()>(13, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'slotClearProject'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotWriteImage'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotWriteImage'
        QtMocHelpers::SlotData<void(const QUrl &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'formatMedium'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 2 },
        }}),
        // Slot 'slotFormatMedium'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'mediaCopy'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 2 },
        }}),
        // Slot 'slotMediaCopy'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cddaRip'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 2 },
        }}),
        // Slot 'slotCddaRip'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'videoDvdRip'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 2 },
        }}),
        // Slot 'slotVideoDvdRip'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'videoCdRip'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 2 },
        }}),
        // Slot 'slotVideoCdRip'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showDiskInfo'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 2 },
        }}),
        // Slot 'slotErrorMessage'
        QtMocHelpers::SlotData<void(const QString &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotWarningMessage'
        QtMocHelpers::SlotData<void(const QString &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotConfigureKeys'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCheckSystem'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotManualCheckSystem'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addUrls'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'slotFileOpen'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFileOpenRecent'
        QtMocHelpers::SlotData<void(const QUrl &)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotFileSave'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFileSaveAs'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFileSaveAll'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFileClose'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFileClose'
        QtMocHelpers::SlotData<void(Doc *)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 44, 45 },
        }}),
        // Slot 'slotFileCloseAll'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSettingsConfigure'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCurrentDocChanged'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFileQuit'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotViewStatusBar'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotStatusMsg'
        QtMocHelpers::SlotData<void(const QString &)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 52 },
        }}),
        // Slot 'slotShowMenuBar'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotProjectAddFiles'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotEditToolbars'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotNewToolBarConfig'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDataImportSession'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDataClearImportedSession'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotEditBootImages'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createClient'
        QtMocHelpers::SlotData<void(K3b::Doc *)>(60, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 45 },
        }}),
        // Slot 'slotCheckSystemTimed'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN3K3b10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<KXmlGuiWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b10MainWindowE_t>.metaTypes,
    nullptr
} };

void K3b::MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->initializationInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->configChanged((*reinterpret_cast<std::add_pointer_t<KSharedConfig::Ptr>>(_a[1]))); break;
        case 2: { K3b::Doc* _r = _t->slotNewAudioDoc();
            if (_a[0]) *reinterpret_cast<K3b::Doc**>(_a[0]) = std::move(_r); }  break;
        case 3: { K3b::Doc* _r = _t->slotNewDataDoc();
            if (_a[0]) *reinterpret_cast<K3b::Doc**>(_a[0]) = std::move(_r); }  break;
        case 4: { K3b::Doc* _r = _t->slotNewMixedDoc();
            if (_a[0]) *reinterpret_cast<K3b::Doc**>(_a[0]) = std::move(_r); }  break;
        case 5: { K3b::Doc* _r = _t->slotNewVcdDoc();
            if (_a[0]) *reinterpret_cast<K3b::Doc**>(_a[0]) = std::move(_r); }  break;
        case 6: { K3b::Doc* _r = _t->slotNewMovixDoc();
            if (_a[0]) *reinterpret_cast<K3b::Doc**>(_a[0]) = std::move(_r); }  break;
        case 7: { K3b::Doc* _r = _t->slotNewVideoDvdDoc();
            if (_a[0]) *reinterpret_cast<K3b::Doc**>(_a[0]) = std::move(_r); }  break;
        case 8: { K3b::Doc* _r = _t->slotContinueMultisession();
            if (_a[0]) *reinterpret_cast<K3b::Doc**>(_a[0]) = std::move(_r); }  break;
        case 9: _t->slotClearProject(); break;
        case 10: _t->slotWriteImage(); break;
        case 11: _t->slotWriteImage((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 12: _t->formatMedium((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 13: _t->slotFormatMedium(); break;
        case 14: _t->mediaCopy((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 15: _t->slotMediaCopy(); break;
        case 16: _t->cddaRip((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 17: _t->slotCddaRip(); break;
        case 18: _t->videoDvdRip((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 19: _t->slotVideoDvdRip(); break;
        case 20: _t->videoCdRip((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 21: _t->slotVideoCdRip(); break;
        case 22: _t->showDiskInfo((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 23: _t->slotErrorMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->slotWarningMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 25: _t->slotConfigureKeys(); break;
        case 26: _t->slotCheckSystem(); break;
        case 27: _t->slotManualCheckSystem(); break;
        case 28: _t->addUrls((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 29: _t->slotFileOpen(); break;
        case 30: _t->slotFileOpenRecent((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 31: _t->slotFileSave(); break;
        case 32: _t->slotFileSaveAs(); break;
        case 33: _t->slotFileSaveAll(); break;
        case 34: _t->slotFileClose(); break;
        case 35: _t->slotFileClose((*reinterpret_cast<std::add_pointer_t<Doc*>>(_a[1]))); break;
        case 36: _t->slotFileCloseAll(); break;
        case 37: _t->slotSettingsConfigure(); break;
        case 38: _t->slotCurrentDocChanged(); break;
        case 39: _t->slotFileQuit(); break;
        case 40: _t->slotViewStatusBar(); break;
        case 41: _t->slotStatusMsg((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 42: _t->slotShowMenuBar(); break;
        case 43: _t->slotProjectAddFiles(); break;
        case 44: _t->slotEditToolbars(); break;
        case 45: _t->slotNewToolBarConfig(); break;
        case 46: _t->slotDataImportSession(); break;
        case 47: _t->slotDataClearImportedSession(); break;
        case 48: _t->slotEditBootImages(); break;
        case 49: _t->createClient((*reinterpret_cast<std::add_pointer_t<K3b::Doc*>>(_a[1]))); break;
        case 50: _t->slotCheckSystemTimed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)(const QString & )>(_a, &MainWindow::initializationInfo, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)(KSharedConfig::Ptr )>(_a, &MainWindow::configChanged, 1))
            return;
    }
}

const QMetaObject *K3b::MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    return KXmlGuiWindow::qt_metacast(_clname);
}

int K3b::MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KXmlGuiWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 51)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 51;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 51)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 51;
    }
    return _id;
}

// SIGNAL 0
void K3b::MainWindow::initializationInfo(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void K3b::MainWindow::configChanged(KSharedConfig::Ptr _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
