/****************************************************************************
** Meta object code from reading C++ file 'k3babstractwriter.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/libk3b/projects/k3babstractwriter.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3babstractwriter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b14AbstractWriterE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::AbstractWriter::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b14AbstractWriterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::AbstractWriter",
        "buffer",
        "",
        "deviceBuffer",
        "writeSpeed",
        "speed",
        "K3b::Device::SpeedMultiplicator",
        "multiplicator",
        "cancel",
        "setBurnDevice",
        "K3b::Device::Device*",
        "dev",
        "setBurnSpeed",
        "s",
        "setSimulate",
        "b",
        "setSourceUnreadable",
        "slotUnblockWhileCancellationFinished",
        "success",
        "slotEjectWhileCancellationFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'buffer'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'deviceBuffer'
        QtMocHelpers::SignalData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'writeSpeed'
        QtMocHelpers::SignalData<void(int, K3b::Device::SpeedMultiplicator)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'cancel'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setBurnDevice'
        QtMocHelpers::SlotData<void(K3b::Device::Device *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setBurnSpeed'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Slot 'setSimulate'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'setSourceUnreadable'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'setSourceUnreadable'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotUnblockWhileCancellationFinished'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Slot 'slotEjectWhileCancellationFinished'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AbstractWriter, qt_meta_tag_ZN3K3b14AbstractWriterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::AbstractWriter::staticMetaObject = { {
    QMetaObject::SuperData::link<Job::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b14AbstractWriterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b14AbstractWriterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b14AbstractWriterE_t>.metaTypes,
    nullptr
} };

void K3b::AbstractWriter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AbstractWriter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->buffer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->deviceBuffer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->writeSpeed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<K3b::Device::SpeedMultiplicator>>(_a[2]))); break;
        case 3: _t->cancel(); break;
        case 4: _t->setBurnDevice((*reinterpret_cast<std::add_pointer_t<K3b::Device::Device*>>(_a[1]))); break;
        case 5: _t->setBurnSpeed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->setSimulate((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->setSourceUnreadable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->setSourceUnreadable(); break;
        case 9: _t->slotUnblockWhileCancellationFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->slotEjectWhileCancellationFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AbstractWriter::*)(int )>(_a, &AbstractWriter::buffer, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractWriter::*)(int )>(_a, &AbstractWriter::deviceBuffer, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractWriter::*)(int , K3b::Device::SpeedMultiplicator )>(_a, &AbstractWriter::writeSpeed, 2))
            return;
    }
}

const QMetaObject *K3b::AbstractWriter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::AbstractWriter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b14AbstractWriterE_t>.strings))
        return static_cast<void*>(this);
    return Job::qt_metacast(_clname);
}

int K3b::AbstractWriter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Job::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void K3b::AbstractWriter::buffer(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void K3b::AbstractWriter::deviceBuffer(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void K3b::AbstractWriter::writeSpeed(int _t1, K3b::Device::SpeedMultiplicator _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
QT_WARNING_POP
