/****************************************************************************
** Meta object code from reading C++ file 'k3bvcddoc.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/libk3b/projects/videocd/k3bvcddoc.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3bvcddoc.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b6VcdDocE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::VcdDoc::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b6VcdDocE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::VcdDoc",
        "aboutToAddVCDTracks",
        "",
        "pos",
        "count",
        "addedVCDTracks",
        "aboutToRemoveVCDTracks",
        "removedVCDTracks",
        "newTracks",
        "trackRemoved",
        "K3b::VcdTrack*",
        "addUrls",
        "QList<QUrl>",
        "addTrack",
        "QUrl",
        "addTracks",
        "track",
        "position",
        "removeTrack",
        "moveTrack",
        "before",
        "slotWorkUrlQueue"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'aboutToAddVCDTracks'
        QtMocHelpers::SignalData<void(int, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 },
        }}),
        // Signal 'addedVCDTracks'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aboutToRemoveVCDTracks'
        QtMocHelpers::SignalData<void(int, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 },
        }}),
        // Signal 'removedVCDTracks'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'newTracks'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'trackRemoved'
        QtMocHelpers::SignalData<void(K3b::VcdTrack *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Slot 'addUrls'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Slot 'addTrack'
        QtMocHelpers::SlotData<void(const QUrl &, uint)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 2 }, { QMetaType::UInt, 2 },
        }}),
        // Slot 'addTracks'
        QtMocHelpers::SlotData<void(const QList<QUrl> &, uint)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 }, { QMetaType::UInt, 2 },
        }}),
        // Slot 'addTrack'
        QtMocHelpers::SlotData<void(K3b::VcdTrack *, uint)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 16 }, { QMetaType::UInt, 17 },
        }}),
        // Slot 'addTrack'
        QtMocHelpers::SlotData<void(K3b::VcdTrack *)>(13, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 10, 16 },
        }}),
        // Slot 'removeTrack'
        QtMocHelpers::SlotData<void(K3b::VcdTrack *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 16 },
        }}),
        // Slot 'moveTrack'
        QtMocHelpers::SlotData<void(K3b::VcdTrack *, K3b::VcdTrack *)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 16 }, { 0x80000000 | 10, 20 },
        }}),
        // Slot 'slotWorkUrlQueue'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VcdDoc, qt_meta_tag_ZN3K3b6VcdDocE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::VcdDoc::staticMetaObject = { {
    QMetaObject::SuperData::link<Doc::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b6VcdDocE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b6VcdDocE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b6VcdDocE_t>.metaTypes,
    nullptr
} };

void K3b::VcdDoc::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<VcdDoc *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->aboutToAddVCDTracks((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->addedVCDTracks(); break;
        case 2: _t->aboutToRemoveVCDTracks((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->removedVCDTracks(); break;
        case 4: _t->newTracks(); break;
        case 5: _t->trackRemoved((*reinterpret_cast<std::add_pointer_t<K3b::VcdTrack*>>(_a[1]))); break;
        case 6: _t->addUrls((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 7: _t->addTrack((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 8: _t->addTracks((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 9: _t->addTrack((*reinterpret_cast<std::add_pointer_t<K3b::VcdTrack*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 10: _t->addTrack((*reinterpret_cast<std::add_pointer_t<K3b::VcdTrack*>>(_a[1]))); break;
        case 11: _t->removeTrack((*reinterpret_cast<std::add_pointer_t<K3b::VcdTrack*>>(_a[1]))); break;
        case 12: _t->moveTrack((*reinterpret_cast<std::add_pointer_t<K3b::VcdTrack*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<K3b::VcdTrack*>>(_a[2]))); break;
        case 13: _t->slotWorkUrlQueue(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (VcdDoc::*)(int , int )>(_a, &VcdDoc::aboutToAddVCDTracks, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (VcdDoc::*)()>(_a, &VcdDoc::addedVCDTracks, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (VcdDoc::*)(int , int )>(_a, &VcdDoc::aboutToRemoveVCDTracks, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (VcdDoc::*)()>(_a, &VcdDoc::removedVCDTracks, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (VcdDoc::*)()>(_a, &VcdDoc::newTracks, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (VcdDoc::*)(K3b::VcdTrack * )>(_a, &VcdDoc::trackRemoved, 5))
            return;
    }
}

const QMetaObject *K3b::VcdDoc::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::VcdDoc::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b6VcdDocE_t>.strings))
        return static_cast<void*>(this);
    return Doc::qt_metacast(_clname);
}

int K3b::VcdDoc::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Doc::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void K3b::VcdDoc::aboutToAddVCDTracks(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void K3b::VcdDoc::addedVCDTracks()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void K3b::VcdDoc::aboutToRemoveVCDTracks(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void K3b::VcdDoc::removedVCDTracks()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void K3b::VcdDoc::newTracks()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void K3b::VcdDoc::trackRemoved(K3b::VcdTrack * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
