/****************************************************************************
** Meta object code from reading C++ file 'k3bprojectmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/src/k3bprojectmanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3bprojectmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b14ProjectManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::ProjectManager::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b14ProjectManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::ProjectManager",
        "newProject",
        "",
        "K3b::Doc*",
        "projectSaved",
        "closingProject",
        "projectChanged",
        "doc",
        "activeProjectChanged",
        "addProject",
        "removeProject",
        "setActive",
        "loadDefaults",
        "slotProjectChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newProject'
        QtMocHelpers::SignalData<void(K3b::Doc *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'projectSaved'
        QtMocHelpers::SignalData<void(K3b::Doc *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'closingProject'
        QtMocHelpers::SignalData<void(K3b::Doc *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'projectChanged'
        QtMocHelpers::SignalData<void(K3b::Doc *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 7 },
        }}),
        // Signal 'activeProjectChanged'
        QtMocHelpers::SignalData<void(K3b::Doc *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'addProject'
        QtMocHelpers::SlotData<void(K3b::Doc *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'removeProject'
        QtMocHelpers::SlotData<void(K3b::Doc *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'setActive'
        QtMocHelpers::SlotData<void(K3b::Doc *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'loadDefaults'
        QtMocHelpers::SlotData<void(K3b::Doc *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'slotProjectChanged'
        QtMocHelpers::SlotData<void(K3b::Doc *)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ProjectManager, qt_meta_tag_ZN3K3b14ProjectManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::ProjectManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b14ProjectManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b14ProjectManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b14ProjectManagerE_t>.metaTypes,
    nullptr
} };

void K3b::ProjectManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ProjectManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newProject((*reinterpret_cast<std::add_pointer_t<K3b::Doc*>>(_a[1]))); break;
        case 1: _t->projectSaved((*reinterpret_cast<std::add_pointer_t<K3b::Doc*>>(_a[1]))); break;
        case 2: _t->closingProject((*reinterpret_cast<std::add_pointer_t<K3b::Doc*>>(_a[1]))); break;
        case 3: _t->projectChanged((*reinterpret_cast<std::add_pointer_t<K3b::Doc*>>(_a[1]))); break;
        case 4: _t->activeProjectChanged((*reinterpret_cast<std::add_pointer_t<K3b::Doc*>>(_a[1]))); break;
        case 5: _t->addProject((*reinterpret_cast<std::add_pointer_t<K3b::Doc*>>(_a[1]))); break;
        case 6: _t->removeProject((*reinterpret_cast<std::add_pointer_t<K3b::Doc*>>(_a[1]))); break;
        case 7: _t->setActive((*reinterpret_cast<std::add_pointer_t<K3b::Doc*>>(_a[1]))); break;
        case 8: _t->loadDefaults((*reinterpret_cast<std::add_pointer_t<K3b::Doc*>>(_a[1]))); break;
        case 9: _t->slotProjectChanged((*reinterpret_cast<std::add_pointer_t<K3b::Doc*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< K3b::Doc* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< K3b::Doc* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< K3b::Doc* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< K3b::Doc* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< K3b::Doc* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< K3b::Doc* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< K3b::Doc* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< K3b::Doc* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< K3b::Doc* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< K3b::Doc* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ProjectManager::*)(K3b::Doc * )>(_a, &ProjectManager::newProject, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProjectManager::*)(K3b::Doc * )>(_a, &ProjectManager::projectSaved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProjectManager::*)(K3b::Doc * )>(_a, &ProjectManager::closingProject, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProjectManager::*)(K3b::Doc * )>(_a, &ProjectManager::projectChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProjectManager::*)(K3b::Doc * )>(_a, &ProjectManager::activeProjectChanged, 4))
            return;
    }
}

const QMetaObject *K3b::ProjectManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::ProjectManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b14ProjectManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int K3b::ProjectManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void K3b::ProjectManager::newProject(K3b::Doc * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void K3b::ProjectManager::projectSaved(K3b::Doc * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void K3b::ProjectManager::closingProject(K3b::Doc * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void K3b::ProjectManager::projectChanged(K3b::Doc * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void K3b::ProjectManager::activeProjectChanged(K3b::Doc * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
