/****************************************************************************
** Meta object code from reading C++ file 'k3bjobinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/src/k3bjobinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3bjobinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b12JobInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::JobInterface::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b12JobInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::JobInterface",
        "D-Bus Interface",
        "org.k3b.Job",
        "started",
        "",
        "canceled",
        "finished",
        "infoMessage",
        "progress",
        "subProgress",
        "newTask",
        "newSubTask",
        "buffer",
        "deviceBuffer",
        "nextTrack",
        "track",
        "numTracks",
        "jobRunning",
        "jobDescription",
        "jobDetails",
        "slotProgress",
        "slotSubProgress"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'started'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'canceled'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(bool)>(6, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Signal 'infoMessage'
        QtMocHelpers::SignalData<void(const QString &, int)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::Int, 4 },
        }}),
        // Signal 'progress'
        QtMocHelpers::SignalData<void(int)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'subProgress'
        QtMocHelpers::SignalData<void(int)>(9, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'newTask'
        QtMocHelpers::SignalData<void(const QString &)>(10, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'newSubTask'
        QtMocHelpers::SignalData<void(const QString &)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'buffer'
        QtMocHelpers::SignalData<void(int)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'deviceBuffer'
        QtMocHelpers::SignalData<void(int)>(13, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'nextTrack'
        QtMocHelpers::SignalData<void(int, int)>(14, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { QMetaType::Int, 16 },
        }}),
        // Slot 'jobRunning'
        QtMocHelpers::SlotData<bool() const>(17, 4, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'jobDescription'
        QtMocHelpers::SlotData<QString() const>(18, 4, QMC::AccessPublic, QMetaType::QString),
        // Slot 'jobDetails'
        QtMocHelpers::SlotData<QString() const>(19, 4, QMC::AccessPublic, QMetaType::QString),
        // Slot 'slotProgress'
        QtMocHelpers::SlotData<void(int)>(20, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'slotSubProgress'
        QtMocHelpers::SlotData<void(int)>(21, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<JobInterface, qt_meta_tag_ZN3K3b12JobInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject K3b::JobInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b12JobInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b12JobInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b12JobInterfaceE_t>.metaTypes,
    nullptr
} };

void K3b::JobInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JobInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->started(); break;
        case 1: _t->canceled(); break;
        case 2: _t->finished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->infoMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->progress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->subProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->newTask((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->newSubTask((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->buffer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->deviceBuffer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->nextTrack((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 11: { bool _r = _t->jobRunning();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { QString _r = _t->jobDescription();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 13: { QString _r = _t->jobDetails();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->slotProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->slotSubProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (JobInterface::*)()>(_a, &JobInterface::started, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (JobInterface::*)()>(_a, &JobInterface::canceled, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (JobInterface::*)(bool )>(_a, &JobInterface::finished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (JobInterface::*)(const QString & , int )>(_a, &JobInterface::infoMessage, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (JobInterface::*)(int )>(_a, &JobInterface::progress, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (JobInterface::*)(int )>(_a, &JobInterface::subProgress, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (JobInterface::*)(const QString & )>(_a, &JobInterface::newTask, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (JobInterface::*)(const QString & )>(_a, &JobInterface::newSubTask, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (JobInterface::*)(int )>(_a, &JobInterface::buffer, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (JobInterface::*)(int )>(_a, &JobInterface::deviceBuffer, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (JobInterface::*)(int , int )>(_a, &JobInterface::nextTrack, 10))
            return;
    }
}

const QMetaObject *K3b::JobInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::JobInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b12JobInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int K3b::JobInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void K3b::JobInterface::started()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void K3b::JobInterface::canceled()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void K3b::JobInterface::finished(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void K3b::JobInterface::infoMessage(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void K3b::JobInterface::progress(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void K3b::JobInterface::subProgress(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void K3b::JobInterface::newTask(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void K3b::JobInterface::newSubTask(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void K3b::JobInterface::buffer(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void K3b::JobInterface::deviceBuffer(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void K3b::JobInterface::nextTrack(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}
QT_WARNING_POP
