//
/* This file is part of the KDE project
   SPDX-FileCopyrightText: 2004 Nicolas GOUTTE <goutte@kde.org>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "KoStoreBase.h"

KoStoreBase::KoStoreBase(void) : m_fileMode(Local), m_window(nullptr)
{
}

KoStoreBase::~KoStoreBase(void)
{
}
