/*
    SPDX-FileCopyrightText: 2010 Michal Malek <michalm@jabster.pl>
    SPDX-FileCopyrightText: 1998-2008 Sebastian Trueg <trueg@k3b.org>
    SPDX-License-Identifier: GPL-2.0-or-later
*/

#ifndef _K3B_OGG_VORBIS_ENCODER_DEFAULTS_H_
#define _K3B_OGG_VORBIS_ENCODER_DEFAULTS_H_

const bool DEFAULT_MANUAL_BITRATE = false;
const int DEFAULT_QUALITY_LEVEL = 4;
const int DEFAULT_BITRATE_UPPER = -1;
const int DEFAULT_BITRATE_NOMINAL = -1;
const int DEFAULT_BITRATE_LOWER = -1;
const int DEFAULT_SAMPLERATE = 44100;

#endif // _K3B_OGG_VORBIS_ENCODER_DEFAULTS_H_
