/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.k3b.Project.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "k3bprojectinterfaceadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class K3bProjectInterfaceAdaptor
 */

K3bProjectInterfaceAdaptor::K3bProjectInterfaceAdaptor(K3b::ProjectInterface *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

K3bProjectInterfaceAdaptor::~K3bProjectInterfaceAdaptor()
{
    // destructor
}

void K3bProjectInterfaceAdaptor::addUrl(const QString &url)
{
    // handle method call org.k3b.Project.addUrl
    parent()->addUrl(url);
}

void K3bProjectInterfaceAdaptor::addUrls(const QStringList &urls)
{
    // handle method call org.k3b.Project.addUrls
    parent()->addUrls(urls);
}

void K3bProjectInterfaceAdaptor::burn()
{
    // handle method call org.k3b.Project.burn
    parent()->burn();
}

bool K3bProjectInterfaceAdaptor::directBurn()
{
    // handle method call org.k3b.Project.directBurn
    return parent()->directBurn();
}

void K3bProjectInterfaceAdaptor::imagePath()
{
    // handle method call org.k3b.Project.imagePath
    parent()->imagePath();
}

int K3bProjectInterfaceAdaptor::length()
{
    // handle method call org.k3b.Project.length
    return parent()->length();
}

QString K3bProjectInterfaceAdaptor::projectType()
{
    // handle method call org.k3b.Project.projectType
    return parent()->projectType();
}

void K3bProjectInterfaceAdaptor::setBurnDevice(const QString &blockdevicename)
{
    // handle method call org.k3b.Project.setBurnDevice
    parent()->setBurnDevice(blockdevicename);
}


#include "moc_k3bprojectinterfaceadaptor.cpp"
