/****************************************************************************
** Meta object code from reading C++ file 'k3bvideodvdrippingjob.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/src/rip/videodvd/k3bvideodvdrippingjob.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3bvideodvdrippingjob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b18VideoDVDRippingJobE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::VideoDVDRippingJob::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b18VideoDVDRippingJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::VideoDVDRippingJob",
        "start",
        "",
        "cancel",
        "setVideoDVD",
        "K3b::VideoDVD::VideoDVD",
        "dvd",
        "setTitles",
        "QList<TitleRipInfo>",
        "titles",
        "setVideoCodec",
        "K3b::VideoDVDTitleTranscodingJob::VideoCodec",
        "codec",
        "setVideoBitrate",
        "bitrate",
        "setTwoPassEncoding",
        "b",
        "setAudioCodec",
        "K3b::VideoDVDTitleTranscodingJob::AudioCodec",
        "setAudioBitrate",
        "setAudioVBR",
        "vbr",
        "setResampleAudioTo44100",
        "setLowPriority",
        "setAutoClipping",
        "slotTranscodingJobFinished",
        "slotDetectClippingJobFinished",
        "slotTranscodingProgress",
        "slotDetectClippingProgress"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'start'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cancel'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setVideoDVD'
        QtMocHelpers::SlotData<void(const K3b::VideoDVD::VideoDVD &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setTitles'
        QtMocHelpers::SlotData<void(const QVector<TitleRipInfo> &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'setVideoCodec'
        QtMocHelpers::SlotData<void(K3b::VideoDVDTitleTranscodingJob::VideoCodec)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'setVideoBitrate'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'setTwoPassEncoding'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'setAudioCodec'
        QtMocHelpers::SlotData<void(K3b::VideoDVDTitleTranscodingJob::AudioCodec)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 12 },
        }}),
        // Slot 'setAudioBitrate'
        QtMocHelpers::SlotData<void(int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'setAudioVBR'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Slot 'setResampleAudioTo44100'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'setLowPriority'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'setAutoClipping'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'slotTranscodingJobFinished'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotDetectClippingJobFinished'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotTranscodingProgress'
        QtMocHelpers::SlotData<void(int)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotDetectClippingProgress'
        QtMocHelpers::SlotData<void(int)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VideoDVDRippingJob, qt_meta_tag_ZN3K3b18VideoDVDRippingJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject K3b::VideoDVDRippingJob::staticMetaObject = { {
    QMetaObject::SuperData::link<Job::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b18VideoDVDRippingJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b18VideoDVDRippingJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b18VideoDVDRippingJobE_t>.metaTypes,
    nullptr
} };

void K3b::VideoDVDRippingJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<VideoDVDRippingJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->start(); break;
        case 1: _t->cancel(); break;
        case 2: _t->setVideoDVD((*reinterpret_cast<std::add_pointer_t<K3b::VideoDVD::VideoDVD>>(_a[1]))); break;
        case 3: _t->setTitles((*reinterpret_cast<std::add_pointer_t<QList<TitleRipInfo>>>(_a[1]))); break;
        case 4: _t->setVideoCodec((*reinterpret_cast<std::add_pointer_t<K3b::VideoDVDTitleTranscodingJob::VideoCodec>>(_a[1]))); break;
        case 5: _t->setVideoBitrate((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->setTwoPassEncoding((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->setAudioCodec((*reinterpret_cast<std::add_pointer_t<K3b::VideoDVDTitleTranscodingJob::AudioCodec>>(_a[1]))); break;
        case 8: _t->setAudioBitrate((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->setAudioVBR((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->setResampleAudioTo44100((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->setLowPriority((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->setAutoClipping((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->slotTranscodingJobFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->slotDetectClippingJobFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->slotTranscodingProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->slotDetectClippingProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *K3b::VideoDVDRippingJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::VideoDVDRippingJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b18VideoDVDRippingJobE_t>.strings))
        return static_cast<void*>(this);
    return Job::qt_metacast(_clname);
}

int K3b::VideoDVDRippingJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Job::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}
QT_WARNING_POP
