/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.k3b.Job.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "k3bjobinterfaceadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class K3bJobInterfaceAdaptor
 */

K3bJobInterfaceAdaptor::K3bJobInterfaceAdaptor(K3b::JobInterface *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

K3bJobInterfaceAdaptor::~K3bJobInterfaceAdaptor()
{
    // destructor
}

QString K3bJobInterfaceAdaptor::jobDescription()
{
    // handle method call org.k3b.Job.jobDescription
    return parent()->jobDescription();
}

QString K3bJobInterfaceAdaptor::jobDetails()
{
    // handle method call org.k3b.Job.jobDetails
    return parent()->jobDetails();
}

bool K3bJobInterfaceAdaptor::jobRunning()
{
    // handle method call org.k3b.Job.jobRunning
    return parent()->jobRunning();
}


#include "moc_k3bjobinterfaceadaptor.cpp"
