/****************************************************************************
** Meta object code from reading C++ file 'k3binterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../k3b-26.04.3/src/k3binterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'k3binterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3K3b9InterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto K3b::Interface::qt_create_metaobjectdata<qt_meta_tag_ZN3K3b9InterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "K3b::Interface",
        "D-Bus Interface",
        "org.k3b.MainWindow",
        "createDataProject",
        "",
        "createAudioProject",
        "createMixedProject",
        "createVcdProject",
        "createMovixProject",
        "createVideoDvdProject",
        "currentProject",
        "openProject",
        "url",
        "projects",
        "copyMedium",
        "dev",
        "formatMedium",
        "writeImage",
        "audioCdRip",
        "videoCdRip",
        "videoDvdRip",
        "addUrls",
        "urls",
        "addUrl",
        "blocked"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'createDataProject'
        QtMocHelpers::SlotData<QString()>(3, 4, QMC::AccessPublic, QMetaType::QString),
        // Slot 'createAudioProject'
        QtMocHelpers::SlotData<QString()>(5, 4, QMC::AccessPublic, QMetaType::QString),
        // Slot 'createMixedProject'
        QtMocHelpers::SlotData<QString()>(6, 4, QMC::AccessPublic, QMetaType::QString),
        // Slot 'createVcdProject'
        QtMocHelpers::SlotData<QString()>(7, 4, QMC::AccessPublic, QMetaType::QString),
        // Slot 'createMovixProject'
        QtMocHelpers::SlotData<QString()>(8, 4, QMC::AccessPublic, QMetaType::QString),
        // Slot 'createVideoDvdProject'
        QtMocHelpers::SlotData<QString()>(9, 4, QMC::AccessPublic, QMetaType::QString),
        // Slot 'currentProject'
        QtMocHelpers::SlotData<QString()>(10, 4, QMC::AccessPublic, QMetaType::QString),
        // Slot 'openProject'
        QtMocHelpers::SlotData<QString(const QString &)>(11, 4, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'projects'
        QtMocHelpers::SlotData<QStringList()>(13, 4, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'copyMedium'
        QtMocHelpers::SlotData<void()>(14, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copyMedium'
        QtMocHelpers::SlotData<void(const QString &)>(14, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'formatMedium'
        QtMocHelpers::SlotData<void()>(16, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'writeImage'
        QtMocHelpers::SlotData<void()>(17, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'writeImage'
        QtMocHelpers::SlotData<void(const QString &)>(17, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'audioCdRip'
        QtMocHelpers::SlotData<void()>(18, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'audioCdRip'
        QtMocHelpers::SlotData<void(const QString &)>(18, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'videoCdRip'
        QtMocHelpers::SlotData<void()>(19, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'videoCdRip'
        QtMocHelpers::SlotData<void(const QString &)>(19, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'videoDvdRip'
        QtMocHelpers::SlotData<void()>(20, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'videoDvdRip'
        QtMocHelpers::SlotData<void(const QString &)>(20, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'addUrls'
        QtMocHelpers::SlotData<void(const QStringList &)>(21, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 22 },
        }}),
        // Slot 'addUrl'
        QtMocHelpers::SlotData<void(const QString &)>(23, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'blocked'
        QtMocHelpers::SlotData<bool() const>(24, 4, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<Interface, qt_meta_tag_ZN3K3b9InterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject K3b::Interface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b9InterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b9InterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3K3b9InterfaceE_t>.metaTypes,
    nullptr
} };

void K3b::Interface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Interface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QString _r = _t->createDataProject();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 1: { QString _r = _t->createAudioProject();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { QString _r = _t->createMixedProject();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 3: { QString _r = _t->createVcdProject();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 4: { QString _r = _t->createMovixProject();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 5: { QString _r = _t->createVideoDvdProject();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 6: { QString _r = _t->currentProject();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 7: { QString _r = _t->openProject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 8: { QStringList _r = _t->projects();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->copyMedium(); break;
        case 10: _t->copyMedium((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->formatMedium(); break;
        case 12: _t->writeImage(); break;
        case 13: _t->writeImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->audioCdRip(); break;
        case 15: _t->audioCdRip((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->videoCdRip(); break;
        case 17: _t->videoCdRip((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->videoDvdRip(); break;
        case 19: _t->videoDvdRip((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->addUrls((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 21: _t->addUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: { bool _r = _t->blocked();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *K3b::Interface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *K3b::Interface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3K3b9InterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int K3b::Interface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 23;
    }
    return _id;
}
QT_WARNING_POP
